@@ -30,7 +30,7 @@
 #define GST_TYPE_VISUAL (gst_visual_get_type())
 #define GST_IS_VISUAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VISUAL))
 #define GST_VISUAL(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VISUAL,GstVisual))
-#define GST_IS_VISUAL_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VISUAL))
+#define GST_IS_VISUAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VISUAL))
 #define GST_VISUAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VISUAL,GstVisualClass))
 #define GST_VISUAL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_VISUAL, GstVisualClass))
 
@@ -48,14 +48,17 @@ struct _GstVisual
   GstPad *sinkpad;
   GstPad *srcpad;
   GstClockTime next_ts;
+  GstSegment segment;
 
   /* libvisual stuff */
-  VisAudio audio;
+  VisAudio *audio;
   VisVideo *video;
   VisActor *actor;
 
   /* audio/video state */
+  gint channels;
   gint rate;                    /* Input samplerate */
+  gint bps;
 
   /* framerate numerator & denominator */
   gint fps_n;
@@ -106,8 +109,13 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
         "width = (int) 16, "
         "depth = (int) 16, "
         "endianness = (int) BYTE_ORDER, "
-        "signed = (boolean) TRUE, "
-        "channels = (int) 2, " "rate = (int) [ 1000, MAX ]")
+        "signed = (boolean) TRUE, " "channels = (int) { 1, 2 }, "
+#if defined(VISUAL_API_VERSION) && VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
+        "rate = (int) { 8000, 11250, 22500, 32000, 44100, 48000, 96000 }"
+#else
+        "rate = (int) [ 1000, MAX ]"
+#endif
+    )
     );
 
 
@@ -225,6 +233,10 @@ gst_visual_clear_actors (GstVisual * visual)
     visual_object_unref (VISUAL_OBJECT (visual->video));
     visual->video = NULL;
   }
+  if (visual->audio) {
+    visual_object_unref (VISUAL_OBJECT (visual->audio));
+    visual->audio = NULL;
+  }
 }
 
 static void
@@ -246,6 +258,7 @@ gst_visual_reset (GstVisual * visual)
 {
   visual->next_ts = -1;
   gst_adapter_clear (visual->adapter);
+  gst_segment_init (&visual->segment, GST_FORMAT_UNDEFINED);
 
   GST_OBJECT_LOCK (visual);
   visual->proportion = 1.0;
@@ -304,7 +317,7 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
   GstStructure *structure;
-  gint depth;
+  gint depth, pitch;
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -323,12 +336,12 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
   visual_video_set_depth (visual->video,
       visual_video_depth_enum_from_value (depth));
   visual_video_set_dimension (visual->video, visual->width, visual->height);
+  pitch = GST_ROUND_UP_4 (visual->width * visual->video->bpp);
+  visual_video_set_pitch (visual->video, pitch);
   visual_actor_video_negotiate (visual->actor, 0, FALSE, FALSE);
 
   /* precalc some values */
-  visual->outsize =
-      visual->video->height * GST_ROUND_UP_4 (visual->video->width) *
-      visual->video->bpp;
+  visual->outsize = visual->video->height * pitch;
   visual->spf =
       gst_util_uint64_scale_int (visual->rate, visual->fps_d, visual->fps_n);
   visual->duration =
@@ -354,12 +367,16 @@ gst_visual_sink_setcaps (GstPad * pad, GstCaps * caps)
 
   structure = gst_caps_get_structure (caps, 0);
 
+  gst_structure_get_int (structure, "channels", &visual->channels);
   gst_structure_get_int (structure, "rate", &visual->rate);
 
+  /* this is how many samples we need to fill one frame at the requested
+   * framerate. */
   if (visual->fps_n != 0) {
     visual->spf =
         gst_util_uint64_scale_int (visual->rate, visual->fps_d, visual->fps_n);
   }
+  visual->bps = visual->channels * sizeof (gint16);
 
   gst_object_unref (visual);
   return TRUE;
@@ -386,9 +403,13 @@ gst_vis_src_negotiate (GstVisual * visual)
     target = gst_caps_copy_nth (intersect, 0);
     gst_caps_unref (intersect);
   } else {
-    target = gst_caps_ref ((GstCaps *) templ);
+    /* need a copy, we'll be modifying it when fixating */
+    target = gst_caps_copy (templ);
   }
 
+  /* fixate in case something is not fixed. This does nothing if the value is
+   * already fixed. For video we always try to fixate to something like
+   * 320x240x30 by convention. */
   structure = gst_caps_get_structure (target, 0);
   gst_structure_fixate_field_nearest_int (structure, "width", 320);
   gst_structure_fixate_field_nearest_int (structure, "height", 240);
@@ -405,7 +426,6 @@ no_format:
     GST_ELEMENT_ERROR (visual, STREAM, FORMAT, (NULL),
         ("could not negotiate output format"));
     gst_caps_unref (intersect);
-    gst_caps_unref (othercaps);
     return FALSE;
   }
 }
@@ -427,6 +447,27 @@ gst_visual_sink_event (GstPad * pad, GstEvent * event)
       gst_visual_reset (visual);
       res = gst_pad_push_event (visual->srcpad, event);
       break;
+    case GST_EVENT_NEWSEGMENT:
+    {
+      GstFormat format;
+      gdouble rate, arate;
+      gint64 start, stop, time;
+      gboolean update;
+
+      /* the newsegment values are used to clip the input samples
+       * and to convert the incomming timestamps to running time so
+       * we can do QoS */
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
+          &start, &stop, &time);
+
+      /* now configure the values */
+      gst_segment_set_newsegment_full (&visual->segment, update,
+          rate, arate, format, start, stop, time);
+
+      /* and forward */
+      res = gst_pad_push_event (visual->srcpad, event);
+      break;
+    }
     default:
       res = gst_pad_push_event (visual->srcpad, event);
       break;
@@ -453,9 +494,16 @@ gst_visual_src_event (GstPad * pad, GstEvent * event)
 
       gst_event_parse_qos (event, &proportion, &diff, &timestamp);
 
+      /* save stuff for the _chain function */
       GST_OBJECT_LOCK (visual);
       visual->proportion = proportion;
-      visual->earliest_time = timestamp + diff;
+      if (diff >= 0)
+        /* we're late, this is a good estimate for next displayable
+         * frame (see part-qos.txt) */
+        visual->earliest_time = timestamp + 2 * diff + visual->duration;
+      else
+        visual->earliest_time = timestamp + diff;
+
       GST_OBJECT_UNLOCK (visual);
 
       res = gst_pad_push_event (visual->sinkpad, event);
@@ -470,11 +518,15 @@ gst_visual_src_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+/* allocate and output buffer, if no format was negotiated, this
+ * function will negotiate one. After calling this function, a
+ * reverse negotiation could have happened. */
 static GstFlowReturn
 get_buffer (GstVisual * visual, GstBuffer ** outbuf)
 {
   GstFlowReturn ret;
 
+  /* we don't know an output format yet, pick one */
   if (GST_PAD_CAPS (visual->srcpad) == NULL) {
     if (!gst_vis_src_negotiate (visual))
       return GST_FLOW_NOT_NEGOTIATED;
@@ -483,17 +535,18 @@ get_buffer (GstVisual * visual, GstBuffer ** outbuf)
   GST_DEBUG_OBJECT (visual, "allocating output buffer with caps %"
       GST_PTR_FORMAT, GST_PAD_CAPS (visual->srcpad));
 
+  /* now allocate a buffer with the last negotiated format. 
+   * Downstream could renegotiate a new format, which will trigger
+   * our setcaps function on the source pad. */
   ret =
       gst_pad_alloc_buffer_and_set_caps (visual->srcpad,
       GST_BUFFER_OFFSET_NONE, visual->outsize,
       GST_PAD_CAPS (visual->srcpad), outbuf);
 
+  /* no buffer allocated, we don't care why. */
   if (ret != GST_FLOW_OK)
     return ret;
 
-  if (*outbuf == NULL)
-    return GST_FLOW_ERROR;
-
   return GST_FLOW_OK;
 }
 
@@ -504,6 +557,7 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
   guint i;
   GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
   GstFlowReturn ret = GST_FLOW_OK;
+  guint avail;
 
   GST_DEBUG_OBJECT (visual, "chain function called");
 
@@ -517,43 +571,139 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     }
   }
 
+  /* resync on DISCONT */
+  if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
+    gst_adapter_clear (visual->adapter);
+    visual->next_ts = -1;
+  }
+
   /* Match timestamps from the incoming audio */
   if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE)
     visual->next_ts = GST_BUFFER_TIMESTAMP (buffer);
 
+  GST_DEBUG_OBJECT (visual,
+      "Input buffer has %d samples, time=%" G_GUINT64_FORMAT,
+      GST_BUFFER_SIZE (buffer) / visual->bps, GST_BUFFER_TIMESTAMP (buffer));
+
   gst_adapter_push (visual->adapter, buffer);
 
-  while (gst_adapter_available (visual->adapter) > MAX (512, visual->spf) * 4 &&
-      (ret == GST_FLOW_OK)) {
+  while (TRUE) {
     gboolean need_skip;
-
-    /* Read 512 samples per channel */
     const guint16 *data;
 
-    data = (const guint16 *) gst_adapter_peek (visual->adapter, 512 * 4);
+    GST_DEBUG_OBJECT (visual, "processing buffer");
 
-    GST_DEBUG_OBJECT (visual, "QoS: in: %" G_GUINT64_FORMAT
-        ", earliest: %" G_GUINT64_FORMAT, visual->next_ts,
-        visual->earliest_time);
+    avail = gst_adapter_available (visual->adapter);
+    GST_DEBUG_OBJECT (visual, "avail now %u", avail);
+
+    /* we need at least 512 samples */
+    if (avail < 512 * visual->bps)
+      break;
+
+    /* we need at least enough samples to make one frame */
+    if (avail < visual->spf * visual->bps)
+      break;
 
     if (visual->next_ts != -1) {
+      gint64 qostime;
+
+      /* QoS is done on running time */
+      qostime = gst_segment_to_running_time (&visual->segment, GST_FORMAT_TIME,
+          visual->next_ts);
+
       GST_OBJECT_LOCK (visual);
       /* check for QoS, don't compute buffers that are known to be late */
       need_skip = visual->earliest_time != -1 &&
-          visual->next_ts <= visual->earliest_time;
+          qostime <= visual->earliest_time;
       GST_OBJECT_UNLOCK (visual);
 
       if (need_skip) {
-        GST_WARNING_OBJECT (visual, "skipping frame because of QoS");
+        GST_WARNING_OBJECT (visual,
+            "QoS: skip ts: %" GST_TIME_FORMAT ", earliest: %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (qostime), GST_TIME_ARGS (visual->earliest_time));
         goto skip;
       }
     }
 
-    for (i = 0; i < 512; i++) {
-      visual->audio.plugpcm[0][i] = *data++;
-      visual->audio.plugpcm[1][i] = *data++;
-    }
+    /* Read 512 samples per channel */
+    data =
+        (const guint16 *) gst_adapter_peek (visual->adapter, 512 * visual->bps);
 
+#if defined(VISUAL_API_VERSION) && VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
+    {
+      VisBuffer *lbuf, *rbuf;
+      guint16 ldata[512], rdata[512];
+      VisAudioSampleRateType rate;
+
+      lbuf = visual_buffer_new_with_buffer (ldata, sizeof (ldata), NULL);
+      rbuf = visual_buffer_new_with_buffer (rdata, sizeof (rdata), NULL);
+
+      if (visual->channels == 2) {
+        for (i = 0; i < 512; i++) {
+          ldata[i] = *data++;
+          rdata[i] = *data++;
+        }
+      } else {
+        for (i = 0; i < 512; i++) {
+          ldata[i] = *data;
+          rdata[i] = *data++;
+        }
+      }
+
+      switch (visual->rate) {
+        case 8000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_8000;
+          break;
+        case 11250:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_11250;
+          break;
+        case 22500:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_22500;
+          break;
+        case 32000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_32000;
+          break;
+        case 44100:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_44100;
+          break;
+        case 48000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_48000;
+          break;
+        case 96000:
+          rate = VISUAL_AUDIO_SAMPLE_RATE_96000;
+          break;
+        default:
+          g_assert_not_reached ();
+          break;
+      }
+
+      visual_audio_samplepool_input_channel (visual->audio->samplepool,
+          lbuf,
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_LEFT);
+      visual_audio_samplepool_input_channel (visual->audio->samplepool,
+          rbuf,
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_RIGHT);
+
+      visual_object_unref (VISUAL_OBJECT (lbuf));
+      visual_object_unref (VISUAL_OBJECT (rbuf));
+
+    }
+#else
+    if (visual->channels == 2) {
+      for (i = 0; i < 512; i++) {
+        visual->audio->plugpcm[0][i] = *data++;
+        visual->audio->plugpcm[1][i] = *data++;
+      }
+    } else {
+      for (i = 0; i < 512; i++) {
+        visual->audio->plugpcm[0][i] = *data;
+        visual->audio->plugpcm[1][i] = *data++;
+      }
+    }
+#endif
+
+    /* alloc a buffer if we don't have one yet, this happens
+     * when we pushed a buffer in this while loop before */
     if (outbuf == NULL) {
       ret = get_buffer (visual, &outbuf);
       if (ret != GST_FLOW_OK) {
@@ -561,9 +711,10 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
       }
     }
     visual_video_set_buffer (visual->video, GST_BUFFER_DATA (outbuf));
-    visual_audio_analyze (&visual->audio);
-    visual_actor_run (visual->actor, &visual->audio);
+    visual_audio_analyze (visual->audio);
+    visual_actor_run (visual->actor, visual->audio);
     visual_video_set_buffer (visual->video, NULL);
+    GST_DEBUG_OBJECT (visual, "rendered one frame");
 
     GST_BUFFER_TIMESTAMP (outbuf) = visual->next_ts;
     GST_BUFFER_DURATION (outbuf) = visual->duration;
@@ -571,16 +722,20 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     ret = gst_pad_push (visual->srcpad, outbuf);
     outbuf = NULL;
 
-    GST_DEBUG_OBJECT (visual, "finished frame, flushing %u samples from input",
-        visual->spf);
   skip:
     /* interpollate next timestamp */
     if (visual->next_ts != -1)
       visual->next_ts += visual->duration;
 
-    /* Flush out the number of samples per frame * channels * sizeof (gint16) */
-    gst_adapter_flush (visual->adapter,
-        MIN (gst_adapter_available (visual->adapter), visual->spf * 4));
+    GST_DEBUG_OBJECT (visual, "finished frame, flushing %u samples from input",
+        visual->spf);
+
+    /* Flush out the number of samples per frame */
+    gst_adapter_flush (visual->adapter, visual->spf * visual->bps);
+
+    /* quit the loop if something was wrong */
+    if (ret != GST_FLOW_OK)
+      break;
   }
 
   if (outbuf != NULL)
@@ -604,7 +759,7 @@ gst_visual_change_state (GstElement * element, GstStateChange transition)
           visual_actor_new (GST_VISUAL_GET_CLASS (visual)->plugin->info->
           plugname);
       visual->video = visual_video_new ();
-
+      visual->audio = visual_audio_new ();
       /* can't have a play without actors */
       if (!visual->actor || !visual->video)
         goto no_actors;
@@ -674,10 +829,40 @@ make_valid_name (char *name)
   }
 }
 
+static gboolean
+gst_visual_actor_plugin_is_gl (VisObject * plugin, const gchar * name)
+{
+  gboolean is_gl;
+  gint depth;
+
+#if !defined(VISUAL_API_VERSION)
+
+  depth = VISUAL_PLUGIN_ACTOR (plugin)->depth;
+  is_gl = (depth == VISUAL_VIDEO_DEPTH_GL);
+
+#elif VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
+
+  depth = VISUAL_ACTOR_PLUGIN (plugin)->vidoptions.depth;
+  /* FIXME: how to figure this out correctly in 0.4? */
+  is_gl = (depth & VISUAL_VIDEO_DEPTH_GL) == VISUAL_VIDEO_DEPTH_GL;
+
+#else
+# error what libvisual version is this?
+#endif
+
+  if (!is_gl) {
+    GST_DEBUG ("plugin %s is not a GL plugin (%d), registering", name, depth);
+  } else {
+    GST_DEBUG ("plugin %s is a GL plugin (%d), ignoring", name, depth);
+  }
+
+  return is_gl;
+}
+
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  guint i;
+  guint i, count;
   VisList *list;
 
   GST_DEBUG_CATEGORY_INIT (libvisual_debug, "libvisual", 0,
@@ -697,11 +882,17 @@ plugin_init (GstPlugin * plugin)
       return FALSE;
 
   list = visual_actor_get_list ();
-  for (i = 0; i < visual_list_count (list); i++) {
+
+#if !defined(VISUAL_API_VERSION)
+  count = visual_list_count (list);
+#elif VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
+  count = visual_collection_size (VISUAL_COLLECTION (list));
+#endif
+
+  for (i = 0; i < count; i++) {
     VisPluginRef *ref = visual_list_get (list, i);
-    VisActorPlugin *actplugin = NULL;
     VisPluginData *visplugin = NULL;
-    gboolean is_gl = FALSE;
+    gboolean skip = FALSE;
     GType type;
     gchar *name;
     GTypeInfo info = {
@@ -721,21 +912,19 @@ plugin_init (GstPlugin * plugin)
     if (ref->info->plugname == NULL)
       continue;
 
-    actplugin = VISUAL_PLUGIN_ACTOR (visplugin->info->plugin);
-
-    /* Ignore plugins that only support GL output for now */
-    if (actplugin->depth == VISUAL_VIDEO_DEPTH_GL) {
-      GST_DEBUG ("plugin %s is a GL plugin (%d), ignoring",
-          ref->info->plugname, actplugin->depth);
-      is_gl = TRUE;
+    /* Blacklist some plugins */
+    if (strcmp (ref->info->plugname, "gstreamer") == 0 ||
+        strcmp (ref->info->plugname, "gdkpixbuf") == 0) {
+      skip = TRUE;
     } else {
-      GST_DEBUG ("plugin %s is not a GL plugin (%d), registering",
-          ref->info->plugname, actplugin->depth);
+      /* Ignore plugins that only support GL output for now */
+      skip = gst_visual_actor_plugin_is_gl (visplugin->info->plugin,
+          visplugin->info->plugname);
     }
 
     visual_plugin_unload (visplugin);
 
-    if (!is_gl) {
+    if (!skip) {
       name = g_strdup_printf ("GstVisual%s", ref->info->plugname);
       make_valid_name (name);
       type = g_type_register_static (GST_TYPE_VISUAL, name, &info, 0);
