@@ -28,11 +28,31 @@
  * video codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
  * Foundation</ulink>, based on the VP3 codec.
  * </para>
+ * <para>
+ * The theora codec internally only supports encoding of images that are a
+ * multiple of 16 pixels in both X and Y direction. It is however perfectly
+ * possible to encode images with other dimensions because an arbitrary
+ * rectangular cropping region can be set up. This element will automatically
+ * set up a correct cropping region if the dimensions are not multiples of 16
+ * pixels. The "border" and "center" properties control how this cropping
+ * region will be set up.
+ * </para>
+ * <para>
+ * To control the quality of the encoding, the "bitrate" and "quality"
+ * properties can be used. These two properties are mutualy exclusive. Setting
+ * the bitrate property will produce a constant bitrate (CBR) stream while
+ * setting the quality property will produce a variable bitrate (VBR) stream.
+ * </para>
  * <title>Example pipeline</title>
  * <programlisting>
  * gst-launch -v videotestsrc num-buffers=1000 ! theoraenc ! oggmux ! filesink location=videotestsrc.ogg
  * </programlisting>
+ * This example pipeline will encode a test video source to theora muxed in an
+ * ogg container. Refer to the theoradec documentation to decode the create
+ * stream.
  * </refsect2>
+ *
+ * Last reviewed on 2006-03-01 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,18 +60,21 @@
 #endif
 
 #include "gsttheoraenc.h"
+
 #include <string.h>
+#include <stdlib.h>             /* free */
+
 #include <gst/tag/tag.h>
 
-GST_DEBUG_CATEGORY (theoraenc_debug);
 #define GST_CAT_DEFAULT theoraenc_debug
+GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 #define GST_TYPE_BORDER_MODE (gst_border_mode_get_type())
 static GType
 gst_border_mode_get_type (void)
 {
   static GType border_mode_type = 0;
-  static GEnumValue border_mode[] = {
+  static const GEnumValue border_mode[] = {
     {BORDER_NONE, "No Border", "none"},
     {BORDER_BLACK, "Black Border", "black"},
     {BORDER_MIRROR, "Mirror image in borders", "mirror"},
@@ -65,9 +88,18 @@ gst_border_mode_get_type (void)
   return border_mode_type;
 }
 
-#define ROUND_UP_2(x) (((x) + 1) & ~1)
-#define ROUND_UP_4(x) (((x) + 3) & ~3)
-#define ROUND_UP_8(x) (((x) + 7) & ~7)
+/* taken from theora/lib/toplevel.c */
+static int
+_ilog (unsigned int v)
+{
+  int ret = 0;
+
+  while (v) {
+    ret++;
+    v >>= 1;
+  }
+  return (ret);
+}
 
 #define THEORA_DEF_CENTER               TRUE
 #define THEORA_DEF_BORDER               BORDER_BLACK
@@ -81,7 +113,6 @@ gst_border_mode_get_type (void)
 #define THEORA_DEF_KEYFRAME_MINDISTANCE 8
 #define THEORA_DEF_NOISE_SENSITIVITY    1
 #define THEORA_DEF_SHARPNESS            0
-
 enum
 {
   ARG_0,
@@ -100,12 +131,30 @@ enum
   /* FILL ME */
 };
 
-static GstElementDetails theora_enc_details = {
-  "TheoraEnc",
-  "Codec/Encoder/Video",
-  "encode raw YUV video to a theora stream",
-  "Wim Taymans <wim@fluendo.com>",
-};
+/* this function does a straight granulepos -> timestamp conversion */
+static GstClockTime
+granulepos_to_timestamp (GstTheoraEnc * theoraenc, ogg_int64_t granulepos)
+{
+  guint64 iframe, pframe;
+  int shift = theoraenc->granule_shift;
+
+  if (granulepos < 0)
+    return GST_CLOCK_TIME_NONE;
+
+  iframe = granulepos >> shift;
+  pframe = granulepos - (iframe << shift);
+
+  /* num and den are 32 bit, so we can safely multiply with GST_SECOND */
+  return gst_util_uint64_scale ((guint64) (iframe + pframe),
+      GST_SECOND * theoraenc->info.fps_denominator,
+      theoraenc->info.fps_numerator);
+}
+
+static const GstElementDetails theora_enc_details =
+GST_ELEMENT_DETAILS ("Theora video encoder",
+    "Codec/Encoder/Video",
+    "encode raw YUV video to a theora stream",
+    "Wim Taymans <wim@fluendo.com>");
 
 static GstStaticPadTemplate theora_enc_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -135,6 +184,7 @@ static void theora_enc_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void theora_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
+static void theora_enc_finalize (GObject * object);
 
 static void
 gst_theora_enc_base_init (gpointer g_class)
@@ -156,6 +206,7 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
 
   gobject_class->set_property = theora_enc_set_property;
   gobject_class->get_property = theora_enc_get_property;
+  gobject_class->finalize = theora_enc_finalize;
 
   g_object_class_install_property (gobject_class, ARG_CENTER,
       g_param_spec_boolean ("center", "Center",
@@ -236,6 +287,45 @@ gst_theora_enc_init (GstTheoraEnc * enc, GstTheoraEncClass * g_class)
   enc->keyframe_mindistance = THEORA_DEF_KEYFRAME_MINDISTANCE;
   enc->noise_sensitivity = THEORA_DEF_NOISE_SENSITIVITY;
   enc->sharpness = THEORA_DEF_SHARPNESS;
+
+  enc->granule_shift = _ilog (enc->info.keyframe_frequency_force - 1);
+  GST_DEBUG_OBJECT (enc,
+      "keyframe_frequency_force is %d, granule shift is %d",
+      enc->info.keyframe_frequency_force, enc->granule_shift);
+  enc->expected_ts = GST_CLOCK_TIME_NONE;
+}
+
+static void
+theora_enc_finalize (GObject * object)
+{
+  GstTheoraEnc *enc = GST_THEORA_ENC (object);
+
+  GST_DEBUG_OBJECT (enc, "Finalizing");
+  theora_clear (&enc->state);
+  theora_comment_clear (&enc->comment);
+  theora_info_clear (&enc->info);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+theora_enc_reset (GstTheoraEnc * enc)
+{
+  theora_clear (&enc->state);
+  theora_encode_init (&enc->state, &enc->info);
+}
+
+static void
+theora_enc_clear (GstTheoraEnc * enc)
+{
+  enc->packetno = 0;
+  enc->bytes_out = 0;
+  enc->granulepos_offset = 0;
+  enc->timestamp_offset = 0;
+
+  enc->next_ts = GST_CLOCK_TIME_NONE;
+  enc->next_discont = FALSE;
+  enc->expected_ts = GST_CLOCK_TIME_NONE;
 }
 
 static gboolean
@@ -251,6 +341,7 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   gst_structure_get_fraction (structure, "framerate", &fps_n, &fps_d);
   par = gst_structure_get_value (structure, "pixel-aspect-ratio");
 
+  theora_info_clear (&enc->info);
   theora_info_init (&enc->info);
   /* Theora has a divisible-by-sixteen restriction for the encoded video size but
    * we can define a visible area using the frame_width/frame_height */
@@ -262,8 +353,8 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   /* center image if needed */
   if (enc->center) {
     /* make sure offset is even, for easier decoding */
-    enc->offset_x = ROUND_UP_2 ((enc->info_width - enc->width) / 2);
-    enc->offset_y = ROUND_UP_2 ((enc->info_height - enc->height) / 2);
+    enc->offset_x = GST_ROUND_UP_2 ((enc->info_width - enc->width) / 2);
+    enc->offset_y = GST_ROUND_UP_2 ((enc->info_height - enc->height) / 2);
   } else {
     enc->offset_x = 0;
     enc->offset_y = 0;
@@ -271,8 +362,8 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   enc->info.offset_x = enc->offset_x;
   enc->info.offset_y = enc->offset_y;
 
-  enc->info.fps_numerator = fps_n;
-  enc->info.fps_denominator = fps_d;
+  enc->info.fps_numerator = enc->fps_n = fps_n;
+  enc->info.fps_denominator = enc->fps_d = fps_d;
   if (par) {
     enc->info.aspect_numerator = gst_value_get_fraction_numerator (par);
     enc->info.aspect_denominator = gst_value_get_fraction_denominator (par);
@@ -298,30 +389,65 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   enc->info.noise_sensitivity = enc->noise_sensitivity;
   enc->info.sharpness = enc->sharpness;
 
-  theora_encode_init (&enc->state, &enc->info);
+  /* as done in theora */
+  enc->granule_shift = _ilog (enc->info.keyframe_frequency_force - 1);
+  GST_DEBUG_OBJECT (enc,
+      "keyframe_frequency_force is %d, granule shift is %d",
+      enc->info.keyframe_frequency_force, enc->granule_shift);
+
+  theora_enc_reset (enc);
+  enc->initialised = TRUE;
+
+  gst_object_unref (enc);
 
   return TRUE;
 }
 
+static guint64
+granulepos_add (guint64 granulepos, guint64 addend, gint shift)
+{
+  guint64 iframe, pframe;
+
+  iframe = granulepos >> shift;
+  pframe = granulepos - (iframe << shift);
+  iframe += addend;
+
+  return (iframe << shift) + pframe;
+}
+
 /* prepare a buffer for transmission by passing data through libtheora */
 static GstFlowReturn
 theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
     GstClockTime timestamp, GstClockTime duration, GstBuffer ** buffer)
 {
   GstBuffer *buf;
-  GstFlowReturn ret;
+  GstFlowReturn ret = GST_FLOW_OK;
 
-  ret = gst_pad_alloc_buffer_and_set_caps (enc->srcpad,
-      GST_BUFFER_OFFSET_NONE, packet->bytes, GST_PAD_CAPS (enc->srcpad), &buf);
-  if (ret != GST_FLOW_OK)
-    goto no_buffer;
+  buf = gst_buffer_new_and_alloc (packet->bytes);
+  if (!buf) {
+    GST_WARNING_OBJECT (enc, "Could not allocate buffer");
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
 
   memcpy (GST_BUFFER_DATA (buf), packet->packet, packet->bytes);
-  GST_BUFFER_OFFSET (buf) = enc->bytes_out;
-  GST_BUFFER_OFFSET_END (buf) = packet->granulepos;
-  GST_BUFFER_TIMESTAMP (buf) = timestamp;
+  gst_buffer_set_caps (buf, GST_PAD_CAPS (enc->srcpad));
+  /* see ext/ogg/README; OFFSET_END takes "our" granulepos, OFFSET its
+   * time representation */
+  GST_BUFFER_OFFSET_END (buf) =
+      granulepos_add (packet->granulepos, enc->granulepos_offset,
+      enc->granule_shift);
+  GST_BUFFER_OFFSET (buf) = granulepos_to_timestamp (enc,
+      GST_BUFFER_OFFSET_END (buf));
+
+  GST_BUFFER_TIMESTAMP (buf) = timestamp + enc->timestamp_offset;
   GST_BUFFER_DURATION (buf) = duration;
 
+  if (enc->next_discont) {
+    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+    enc->next_discont = FALSE;
+  }
+
   /* the second most significant bit of the first data byte is cleared
    * for keyframes */
   if ((packet->packet[0] & 0x40) == 0) {
@@ -331,14 +457,9 @@ theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
   }
   enc->packetno++;
 
+done:
   *buffer = buf;
   return ret;
-
-no_buffer:
-  {
-    *buffer = NULL;
-    return ret;
-  }
 }
 
 /* push out the buffer and do internal bookkeeping */
@@ -384,23 +505,38 @@ theora_set_header_on_caps (GstCaps * caps, GstBuffer * buf1,
   GST_BUFFER_FLAG_SET (buf2, GST_BUFFER_FLAG_IN_CAPS);
   GST_BUFFER_FLAG_SET (buf3, GST_BUFFER_FLAG_IN_CAPS);
 
-  /* put buffers in a fixed list */
+  /* Copy buffers, because we can't use the originals -
+   * it creates a circular refcount with the caps<->buffers */
+  buf1 = gst_buffer_copy (buf1);
+  buf2 = gst_buffer_copy (buf2);
+  buf3 = gst_buffer_copy (buf3);
+
+  /* put copies of the buffers in a fixed list */
   g_value_init (&array, GST_TYPE_ARRAY);
+
   g_value_init (&value, GST_TYPE_BUFFER);
   gst_value_set_buffer (&value, buf1);
   gst_value_array_append_value (&array, &value);
   g_value_unset (&value);
+
   g_value_init (&value, GST_TYPE_BUFFER);
   gst_value_set_buffer (&value, buf2);
   gst_value_array_append_value (&array, &value);
   g_value_unset (&value);
+
   g_value_init (&value, GST_TYPE_BUFFER);
   gst_value_set_buffer (&value, buf3);
   gst_value_array_append_value (&array, &value);
-  gst_structure_set_value (structure, "streamheader", &array);
   g_value_unset (&value);
+
+  gst_structure_set_value (structure, "streamheader", &array);
   g_value_unset (&array);
 
+  /* Unref our copies */
+  gst_buffer_unref (buf1);
+  gst_buffer_unref (buf2);
+  gst_buffer_unref (buf3);
+
   return caps;
 }
 
@@ -415,12 +551,17 @@ theora_enc_sink_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
-      /* push last packet with eos flag */
-      while (theora_encode_packetout (&enc->state, 1, &op)) {
-        GstClockTime out_time =
-            theora_granule_time (&enc->state, op.granulepos) * GST_SECOND;
+      if (enc->initialised) {
+        /* push last packet with eos flag */
+        while (theora_encode_packetout (&enc->state, 1, &op)) {
+          /* See comment in the chain function */
+          GstClockTime next_time = theora_granule_time (&enc->state,
+              granulepos_add (op.granulepos, 1, enc->granule_shift)) *
+              GST_SECOND;
 
-        theora_push_packet (enc, &op, out_time, GST_SECOND / enc->fps);
+          theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
+          enc->next_ts = next_time;
+        }
       }
       res = gst_pad_push_event (enc->srcpad, event);
       break;
@@ -430,6 +571,35 @@ theora_enc_sink_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static gboolean
+theora_enc_is_discontinuous (GstTheoraEnc * enc, GstBuffer * buffer)
+{
+  GstClockTime ts = GST_BUFFER_TIMESTAMP (buffer);
+  GstClockTimeDiff max_diff;
+  gboolean ret = FALSE;
+
+  /* Allow 3/4 a frame off */
+  max_diff = (enc->info.fps_denominator * GST_SECOND * 3) /
+      (enc->info.fps_numerator * 4);
+
+  if (ts != GST_CLOCK_TIME_NONE && enc->expected_ts != GST_CLOCK_TIME_NONE) {
+    if ((GstClockTimeDiff) (ts - enc->expected_ts) > max_diff) {
+      GST_DEBUG_OBJECT (enc, "Incoming TS %" GST_TIME_FORMAT
+          " exceeds expected value %" GST_TIME_FORMAT
+          " by too much, marking discontinuity",
+          GST_TIME_ARGS (ts), GST_TIME_ARGS (enc->expected_ts));
+      ret = TRUE;
+    }
+  }
+
+  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (buffer)))
+    enc->expected_ts = ts + GST_BUFFER_DURATION (buffer);
+  else
+    enc->expected_ts = GST_CLOCK_TIME_NONE;
+
+  return ret;
+}
+
 static GstFlowReturn
 theora_enc_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -442,11 +612,15 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
 
   in_time = GST_BUFFER_TIMESTAMP (buffer);
 
-  /* no packets written yet, setup headers */
   if (enc->packetno == 0) {
+    /* no packets written yet, setup headers */
     GstCaps *caps;
     GstBuffer *buf1, *buf2, *buf3;
 
+    enc->granulepos_offset = 0;
+    enc->timestamp_offset = 0;
+
+    GST_DEBUG_OBJECT (enc, "output headers");
     /* Theora streams begin with three headers; the initial header (with
        most of the codec setup parameters) which is mandated by the Ogg
        bitstream spec.  The second header holds any comment fields.  The
@@ -455,26 +629,41 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
        libtheora handles the additional Ogg bitstream constraints */
 
     /* first packet will get its own page automatically */
-    theora_encode_header (&enc->state, &op);
-    ret = theora_buffer_from_packet (enc, &op, 0, 0, &buf1);
+    if (theora_encode_header (&enc->state, &op) != 0)
+      goto encoder_disabled;
+
+    ret = theora_buffer_from_packet (enc, &op, GST_CLOCK_TIME_NONE,
+        GST_CLOCK_TIME_NONE, &buf1);
     if (ret != GST_FLOW_OK) {
       goto header_buffer_alloc;
     }
 
     /* create the remaining theora headers */
+    theora_comment_clear (&enc->comment);
     theora_comment_init (&enc->comment);
-    /* Currently leaks due to libtheora API brokenness, I don't think we can
-     * portably work around it. Leaks ~50 bytes per encoder instance, so not a
-     * huge problem. */
-    theora_encode_comment (&enc->comment, &op);
-    ret = theora_buffer_from_packet (enc, &op, 0, 0, &buf2);
+
+    if (theora_encode_comment (&enc->comment, &op) != 0)
+      goto encoder_disabled;
+
+    ret = theora_buffer_from_packet (enc, &op, GST_CLOCK_TIME_NONE,
+        GST_CLOCK_TIME_NONE, &buf2);
+    /* Theora expects us to put this packet buffer into an ogg page,
+     * in which case it becomes the ogg library's responsibility to
+     * free it. Since we're copying and outputting a gst_buffer,
+     * we need to free it ourselves. */
+    if (op.packet)
+      free (op.packet);
+
     if (ret != GST_FLOW_OK) {
       gst_buffer_unref (buf1);
       goto header_buffer_alloc;
     }
 
-    theora_encode_tables (&enc->state, &op);
-    ret = theora_buffer_from_packet (enc, &op, 0, 0, &buf3);
+    if (theora_encode_tables (&enc->state, &op) != 0)
+      goto encoder_disabled;
+
+    ret = theora_buffer_from_packet (enc, &op, GST_CLOCK_TIME_NONE,
+        GST_CLOCK_TIME_NONE, &buf3);
     if (ret != GST_FLOW_OK) {
       gst_buffer_unref (buf1);
       gst_buffer_unref (buf2);
@@ -491,6 +680,8 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     gst_buffer_set_caps (buf2, caps);
     gst_buffer_set_caps (buf3, caps);
 
+    gst_caps_unref (caps);
+
     /* push out the header buffers */
     if ((ret = theora_push_buffer (enc, buf1)) != GST_FLOW_OK) {
       gst_buffer_unref (buf2);
@@ -504,6 +695,12 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     if ((ret = theora_push_buffer (enc, buf3)) != GST_FLOW_OK) {
       goto header_push;
     }
+
+    enc->granulepos_offset =
+        gst_util_uint64_scale (GST_BUFFER_TIMESTAMP (buffer), enc->fps_n,
+        GST_SECOND * enc->fps_d);
+    enc->timestamp_offset = GST_BUFFER_TIMESTAMP (buffer);
+    enc->next_ts = 0;
   }
 
   {
@@ -523,6 +720,7 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     y_size = enc->info_width * enc->info_height;
 
     if (enc->width == enc->info_width && enc->height == enc->info_height) {
+      GST_LOG_OBJECT (enc, "no cropping/conversion needed");
       /* easy case, no cropping/conversion needed */
       pixels = GST_BUFFER_DATA (buffer);
 
@@ -541,6 +739,7 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       gint cwidth, cheight;
       gint offset_x, right_x, right_border;
 
+      GST_LOG_OBJECT (enc, "cropping/conversion needed for strides");
       /* source width/height */
       width = enc->width;
       height = enc->height;
@@ -549,26 +748,28 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       cheight = height / 2;
 
       /* source strides as defined in videotestsrc */
-      src_y_stride = ROUND_UP_4 (width);
-      src_uv_stride = ROUND_UP_8 (width) / 2;
+      src_y_stride = GST_ROUND_UP_4 (width);
+      src_uv_stride = GST_ROUND_UP_8 (width) / 2;
 
       /* destination strides from the real picture width */
       dst_y_stride = enc->info_width;
       dst_uv_stride = enc->info_width / 2;
 
-      ret = gst_pad_alloc_buffer_and_set_caps (enc->srcpad,
-          GST_BUFFER_OFFSET_NONE, y_size * 3 / 2, GST_PAD_CAPS (enc->srcpad),
-          &newbuf);
-      if (ret != GST_FLOW_OK)
+      newbuf = gst_buffer_new_and_alloc (y_size * 3 / 2);
+      if (!newbuf) {
+        ret = GST_FLOW_ERROR;
         goto no_buffer;
+      }
+      GST_BUFFER_OFFSET (newbuf) = GST_BUFFER_OFFSET_NONE;
+      gst_buffer_set_caps (newbuf, GST_PAD_CAPS (enc->srcpad));
 
       dest_y = yuv.y = GST_BUFFER_DATA (newbuf);
       dest_u = yuv.u = yuv.y + y_size;
       dest_v = yuv.v = yuv.u + y_size / 4;
 
       src_y = GST_BUFFER_DATA (buffer);
-      src_u = src_y + src_y_stride * ROUND_UP_2 (height);
-      src_v = src_u + src_uv_stride * ROUND_UP_2 (height) / 2;
+      src_u = src_y + src_y_stride * GST_ROUND_UP_2 (height);
+      src_v = src_u + src_uv_stride * GST_ROUND_UP_2 (height) / 2;
 
       if (enc->border != BORDER_NONE) {
         /* fill top border */
@@ -651,15 +852,32 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       buffer = newbuf;
     }
 
+    if (theora_enc_is_discontinuous (enc, buffer)) {
+      theora_enc_reset (enc);
+      enc->granulepos_offset =
+          gst_util_uint64_scale (GST_BUFFER_TIMESTAMP (buffer), enc->fps_n,
+          GST_SECOND * enc->fps_d);
+      enc->timestamp_offset = GST_BUFFER_TIMESTAMP (buffer);
+      enc->next_ts = 0;
+      enc->next_discont = TRUE;
+    }
+
     res = theora_encode_YUVin (&enc->state, &yuv);
 
     ret = GST_FLOW_OK;
     while (theora_encode_packetout (&enc->state, 0, &op)) {
-      GstClockTime out_time;
+      /* This is where we hack around theora's broken idea of what granulepos
+         is -- normally we wouldn't need to add the 1, because granulepos
+         should be the presentation time of the last sample in the packet, but
+         theora starts with 0 instead of 1... */
+      GstClockTime next_time;
 
-      out_time = theora_granule_time (&enc->state, op.granulepos) * GST_SECOND;
-      if ((ret = theora_push_packet (enc, &op, out_time, GST_SECOND / enc->fps))
-          != GST_FLOW_OK)
+      next_time = theora_granule_time (&enc->state,
+          granulepos_add (op.granulepos, 1, enc->granule_shift)) * GST_SECOND;
+      ret =
+          theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
+      enc->next_ts = next_time;
+      if (ret != GST_FLOW_OK)
         goto data_push;
     }
     gst_buffer_unref (buffer);
@@ -680,7 +898,6 @@ header_push:
   }
 no_buffer:
   {
-    gst_buffer_unref (buffer);
     return ret;
   }
 data_push:
@@ -688,6 +905,13 @@ data_push:
     gst_buffer_unref (buffer);
     return ret;
   }
+encoder_disabled:
+  {
+    GST_ELEMENT_ERROR (enc, STREAM, ENCODE, (NULL),
+        ("libtheora has been compiled with the encoder disabled"));
+    gst_buffer_unref (buffer);
+    return GST_FLOW_ERROR;
+  }
 }
 
 static GstStateChangeReturn
@@ -702,10 +926,10 @@ theora_enc_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      GST_DEBUG_OBJECT (enc, "READY->PAUSED Initing theora state");
       theora_info_init (&enc->info);
       theora_comment_init (&enc->comment);
       enc->packetno = 0;
-      enc->initial_delay = 0;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -719,9 +943,13 @@ theora_enc_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      GST_DEBUG_OBJECT (enc, "PAUSED->READY Clearing theora state");
       theora_clear (&enc->state);
       theora_comment_clear (&enc->comment);
       theora_info_clear (&enc->info);
+
+      theora_enc_clear (enc);
+      enc->initialised = FALSE;
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
