@@ -137,16 +137,6 @@ copy_and_clean_caps (const GstCaps * caps)
   gst_structure_remove_field (s, "height");
   gst_structure_remove_field (s, "channels");
   gst_structure_remove_field (s, "rate");
-  /* rtp fields */
-  gst_structure_remove_field (s, "config");
-  gst_structure_remove_field (s, "clock-rate");
-  gst_structure_remove_field (s, "clock-base");
-  gst_structure_remove_field (s, "maxps");
-  gst_structure_remove_field (s, "seqnum-base");
-  gst_structure_remove_field (s, "npt-start");
-  gst_structure_remove_field (s, "npt-stop");
-  gst_structure_remove_field (s, "play-speed");
-  gst_structure_remove_field (s, "play-scale");
 
   return ret;
 }
@@ -202,9 +192,9 @@ gst_missing_uri_sink_message_new (GstElement * element, const gchar * protocol)
   GstStructure *s;
   gchar *description;
 
-  g_return_val_if_fail (element != NULL, NULL);
-  g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
-  g_return_val_if_fail (protocol != NULL, NULL);
+  g_return_val_if_fail (element != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (protocol != NULL, FALSE);
 
   description = gst_pb_utils_get_sink_description (protocol);
 
@@ -235,9 +225,9 @@ gst_missing_element_message_new (GstElement * element,
   GstStructure *s;
   gchar *description;
 
-  g_return_val_if_fail (element != NULL, NULL);
-  g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
-  g_return_val_if_fail (factory_name != NULL, NULL);
+  g_return_val_if_fail (element != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (factory_name != NULL, FALSE);
 
   description = gst_pb_utils_get_element_description (factory_name);
 
@@ -268,13 +258,13 @@ gst_missing_decoder_message_new (GstElement * element,
   GstCaps *caps;
   gchar *description;
 
-  g_return_val_if_fail (element != NULL, NULL);
-  g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
-  g_return_val_if_fail (decode_caps != NULL, NULL);
-  g_return_val_if_fail (GST_IS_CAPS (decode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_any (decode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_empty (decode_caps), NULL);
-  g_return_val_if_fail (gst_caps_is_fixed (decode_caps), NULL);
+  g_return_val_if_fail (element != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (decode_caps != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_CAPS (decode_caps), FALSE);
+  g_return_val_if_fail (!gst_caps_is_any (decode_caps), FALSE);
+  g_return_val_if_fail (!gst_caps_is_empty (decode_caps), FALSE);
+  g_return_val_if_fail (gst_caps_is_fixed (decode_caps), FALSE);
 
   description = gst_pb_utils_get_decoder_description (decode_caps);
   caps = copy_and_clean_caps (decode_caps);
@@ -308,13 +298,13 @@ gst_missing_encoder_message_new (GstElement * element,
   GstCaps *caps;
   gchar *description;
 
-  g_return_val_if_fail (element != NULL, NULL);
-  g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
-  g_return_val_if_fail (encode_caps != NULL, NULL);
-  g_return_val_if_fail (GST_IS_CAPS (encode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_any (encode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_empty (encode_caps), NULL);
-  g_return_val_if_fail (gst_caps_is_fixed (encode_caps), NULL);
+  g_return_val_if_fail (element != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (encode_caps != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_CAPS (encode_caps), FALSE);
+  g_return_val_if_fail (!gst_caps_is_any (encode_caps), FALSE);
+  g_return_val_if_fail (!gst_caps_is_empty (encode_caps), FALSE);
+  g_return_val_if_fail (gst_caps_is_fixed (encode_caps), FALSE);
 
   description = gst_pb_utils_get_encoder_description (encode_caps);
   caps = copy_and_clean_caps (encode_caps);
@@ -383,11 +373,11 @@ missing_structure_get_caps_detail (const GstStructure * s, GstCaps ** p_caps)
  * @msg: a missing-plugin #GstMessage of type #GST_MESSAGE_ELEMENT
  *
  * Returns an opaque string containing all the details about the missing
- * element to be passed to an external installer called via
- * gst_install_plugins_async() or gst_install_plugins_sync().
+ * element to be passed to an external installer or installer library
+ * such as libgimme-codec.
  * 
- * This function is mainly for applications that call external plugin
- * installation mechanisms using one of the two above-mentioned functions.
+ * This function is mainly for applications that use libgimme-codec or
+ * other libraries that call external plugin installation mechanisms.
  *
  * Returns: a newly-allocated detail string, or NULL on error. Free string
  *          with g_free() when not needed any longer.
@@ -415,7 +405,6 @@ gst_missing_plugin_message_get_installer_detail (GstMessage * msg)
   type = gst_structure_get_string (msg->structure, "type");
   g_assert (type != NULL);      /* validity already checked above */
 
-  /* FIXME: use gst_installer_detail_new() here too */
   str = g_string_new (GST_DETAIL_STRING_MARKER "|");
   g_string_append_printf (str, "%u.%u|", GST_VERSION_MAJOR, GST_VERSION_MINOR);
 
@@ -423,7 +412,7 @@ gst_missing_plugin_message_get_installer_detail (GstMessage * msg)
   if (progname) {
     g_string_append_printf (str, "%s|", progname);
   } else {
-    g_string_append_printf (str, "pid/%lu|", (gulong) getpid ());
+    g_string_append_printf (str, "pid/%lu", (gulong) getpid ());
   }
 
   desc = gst_missing_plugin_message_get_description (msg);
@@ -454,7 +443,8 @@ gst_missing_plugin_message_get_installer_detail (GstMessage * msg)
       break;
     }
     default:
-      g_return_val_if_reached (NULL);
+      g_assert_not_reached ();
+      break;
   }
 
   g_string_append_printf (str, "%s-%s", type, detail);
@@ -594,204 +584,3 @@ gst_is_missing_plugin_message (GstMessage * msg)
 
   return gst_structure_has_name (msg->structure, "missing-plugin");
 }
-
-/* takes ownership of the description */
-static gchar *
-gst_installer_detail_new (gchar * description, const gchar * type,
-    const gchar * detail)
-{
-  const gchar *progname;
-  GString *s;
-
-  s = g_string_new (GST_DETAIL_STRING_MARKER "|");
-  g_string_append_printf (s, "%u.%u|", GST_VERSION_MAJOR, GST_VERSION_MINOR);
-
-  progname = (const gchar *) g_get_prgname ();
-  if (progname) {
-    g_string_append_printf (s, "%s|", progname);
-  } else {
-    g_string_append_printf (s, "pid/%lu|", (gulong) getpid ());
-  }
-
-  if (description) {
-    g_strdelimit (description, "|", '#');
-    g_string_append_printf (s, "%s|", description);
-    g_free (description);
-  } else {
-    g_string_append (s, "|");
-  }
-
-  g_string_append_printf (s, "%s-%s", type, detail);
-
-  return g_string_free (s, FALSE);
-}
-
-/**
- * gst_missing_uri_source_installer_detail_new:
- * @protocol: the URI protocol the missing source needs to implement,
- *            e.g. "http" or "mms"
- *
- * Returns an opaque string containing all the details about the missing
- * element to be passed to an external installer called via
- * gst_install_plugins_async() or gst_install_plugins_sync().
- * 
- * This function is mainly for applications that call external plugin
- * installation mechanisms using one of the two above-mentioned functions in
- * the case where the application knows exactly what kind of plugin it is
- * missing.
- *
- * Returns: a newly-allocated detail string, or NULL on error. Free string
- *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
- */
-gchar *
-gst_missing_uri_source_installer_detail_new (const gchar * protocol)
-{
-  gchar *desc;
-
-  g_return_val_if_fail (protocol != NULL, NULL);
-
-  desc = gst_pb_utils_get_source_description (protocol);
-  return gst_installer_detail_new (desc, "urisource", protocol);
-}
-
-/**
- * gst_missing_uri_sink_installer_detail_new:
- * @protocol: the URI protocol the missing source needs to implement,
- *            e.g. "http" or "mms"
- *
- * Returns an opaque string containing all the details about the missing
- * element to be passed to an external installer called via
- * gst_install_plugins_async() or gst_install_plugins_sync().
- * 
- * This function is mainly for applications that call external plugin
- * installation mechanisms using one of the two above-mentioned functions in
- * the case where the application knows exactly what kind of plugin it is
- * missing.
- *
- * Returns: a newly-allocated detail string, or NULL on error. Free string
- *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
- */
-gchar *
-gst_missing_uri_sink_installer_detail_new (const gchar * protocol)
-{
-  gchar *desc;
-
-  g_return_val_if_fail (protocol != NULL, NULL);
-
-  desc = gst_pb_utils_get_sink_description (protocol);
-  return gst_installer_detail_new (desc, "urisink", protocol);
-}
-
-/**
- * gst_missing_element_installer_detail_new:
- * @factory_name: the name of the missing element (element factory),
- *            e.g. "videoscale" or "cdparanoiasrc"
- *
- * Returns an opaque string containing all the details about the missing
- * element to be passed to an external installer called via
- * gst_install_plugins_async() or gst_install_plugins_sync().
- * 
- * This function is mainly for applications that call external plugin
- * installation mechanisms using one of the two above-mentioned functions in
- * the case where the application knows exactly what kind of plugin it is
- * missing.
- *
- * Returns: a newly-allocated detail string, or NULL on error. Free string
- *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
- */
-gchar *
-gst_missing_element_installer_detail_new (const gchar * factory_name)
-{
-  gchar *desc;
-
-  g_return_val_if_fail (factory_name != NULL, NULL);
-
-  desc = gst_pb_utils_get_element_description (factory_name);
-  return gst_installer_detail_new (desc, "element", factory_name);
-}
-
-/**
- * gst_missing_decoder_installer_detail_new:
- * @decode_caps: the (fixed) caps for which a decoder element is needed
- *
- * Returns an opaque string containing all the details about the missing
- * element to be passed to an external installer called via
- * gst_install_plugins_async() or gst_install_plugins_sync().
- * 
- * This function is mainly for applications that call external plugin
- * installation mechanisms using one of the two above-mentioned functions in
- * the case where the application knows exactly what kind of plugin it is
- * missing.
- *
- * Returns: a newly-allocated detail string, or NULL on error. Free string
- *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
- */
-gchar *
-gst_missing_decoder_installer_detail_new (const GstCaps * decode_caps)
-{
-  GstCaps *caps;
-  gchar *detail_str, *caps_str, *desc;
-
-  g_return_val_if_fail (decode_caps != NULL, NULL);
-  g_return_val_if_fail (GST_IS_CAPS (decode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_any (decode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_empty (decode_caps), NULL);
-  g_return_val_if_fail (gst_caps_is_fixed (decode_caps), NULL);
-
-  desc = gst_pb_utils_get_decoder_description (decode_caps);
-  caps = copy_and_clean_caps (decode_caps);
-  caps_str = gst_caps_to_string (caps);
-  detail_str = gst_installer_detail_new (desc, "decoder", caps_str);
-  g_free (caps_str);
-  gst_caps_unref (caps);
-
-  return detail_str;
-}
-
-/**
- * gst_missing_encoder_installer_detail_new:
- * @encode_caps: the (fixed) caps for which an encoder element is needed
- *
- * Returns an opaque string containing all the details about the missing
- * element to be passed to an external installer called via
- * gst_install_plugins_async() or gst_install_plugins_sync().
- * 
- * This function is mainly for applications that call external plugin
- * installation mechanisms using one of the two above-mentioned functions in
- * the case where the application knows exactly what kind of plugin it is
- * missing.
- *
- * Returns: a newly-allocated detail string, or NULL on error. Free string
- *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
- */
-gchar *
-gst_missing_encoder_installer_detail_new (const GstCaps * encode_caps)
-{
-  GstCaps *caps;
-  gchar *detail_str, *caps_str, *desc;
-
-  g_return_val_if_fail (encode_caps != NULL, NULL);
-  g_return_val_if_fail (GST_IS_CAPS (encode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_any (encode_caps), NULL);
-  g_return_val_if_fail (!gst_caps_is_empty (encode_caps), NULL);
-  g_return_val_if_fail (gst_caps_is_fixed (encode_caps), NULL);
-
-  desc = gst_pb_utils_get_encoder_description (encode_caps);
-  caps = copy_and_clean_caps (encode_caps);
-  caps_str = gst_caps_to_string (caps);
-  detail_str = gst_installer_detail_new (desc, "encoder", caps_str);
-  g_free (caps_str);
-  gst_caps_unref (caps);
-
-  return detail_str;
-}
