@@ -29,6 +29,7 @@
 #include <gst/audio/multichannel.h>
 
 #include <string.h>
+#include <math.h>
 
 GST_DEBUG_CATEGORY_EXTERN (riff_debug);
 #define GST_CAT_DEFAULT riff_debug
@@ -55,6 +56,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
   GstCaps *caps = NULL;
   GstBuffer *palette = NULL;
 
+  GST_DEBUG ("video fourcc %" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (codec_fcc));
+
   switch (codec_fcc) {
     case GST_MAKE_FOURCC ('D', 'I', 'B', ' '):{
       gint bpp = (strf && strf->bit_cnt != 0) ? strf->bit_cnt : 8;
@@ -174,58 +177,68 @@ gst_riff_create_video_caps (guint32 codec_fcc,
     case GST_MAKE_FOURCC ('h', '2', '6', '3'):
     case GST_MAKE_FOURCC ('i', '2', '6', '3'):
     case GST_MAKE_FOURCC ('U', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "itu", NULL);
       if (codec_name)
         *codec_name = g_strdup ("ITU H.26n");
       break;
 
     case GST_MAKE_FOURCC ('L', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      /* http://www.leadcodecs.com/Codecs/LEAD-H263.htm */
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "lead", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Lead H.263");
       break;
 
     case GST_MAKE_FOURCC ('M', '2', '6', '3'):
     case GST_MAKE_FOURCC ('m', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "microsoft", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Microsoft H.263");
       break;
 
     case GST_MAKE_FOURCC ('V', 'D', 'O', 'W'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "vdolive", NULL);
       if (codec_name)
         *codec_name = g_strdup ("VDOLive");
       break;
 
     case GST_MAKE_FOURCC ('V', 'I', 'V', 'O'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "vivo", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Vivo H.263");
       break;
 
     case GST_MAKE_FOURCC ('x', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-h263",
+          "variant", G_TYPE_STRING, "xirlink", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Xirlink H.263");
       break;
 
       /* apparently not standard H.263...? */
     case GST_MAKE_FOURCC ('I', '2', '6', '3'):
-      caps = gst_caps_new_simple ("video/x-intel-h263", NULL);
+      caps = gst_caps_new_simple ("video/x-intel-h263",
+          "variant", G_TYPE_STRING, "intel", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Intel H.263");
       break;
 
     case GST_MAKE_FOURCC ('H', '2', '6', '4'):
     case GST_MAKE_FOURCC ('h', '2', '6', '4'):
-      caps = gst_caps_new_simple ("video/x-h264", NULL);
+      caps = gst_caps_new_simple ("video/x-h264",
+          "variant", G_TYPE_STRING, "itu", NULL);
       if (codec_name)
         *codec_name = g_strdup ("ITU H.264");
       break;
 
     case GST_MAKE_FOURCC ('V', 'S', 'S', 'H'):
-      caps = gst_caps_new_simple ("video/x-h264", NULL);
+      caps = gst_caps_new_simple ("video/x-h264",
+          "variant", G_TYPE_STRING, "videosoft", NULL);
       if (codec_name)
         *codec_name = g_strdup ("VideoSoft H.264");
       break;
@@ -369,6 +382,14 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Microsoft Windows Media 9");
       break;
 
+    case GST_MAKE_FOURCC ('W', 'M', 'V', 'A'):
+      caps = gst_caps_new_simple ("video/x-wmv",
+          "wmvversion", G_TYPE_INT, 3, "fourcc", GST_TYPE_FOURCC,
+          codec_fcc, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Microsoft Windows Media Advanced Profile");
+      break;
+
     case GST_MAKE_FOURCC ('c', 'v', 'i', 'd'):
       caps = gst_caps_new_simple ("video/x-cinepak", NULL);
       if (codec_name)
@@ -501,11 +522,20 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('T', 'S', 'C', 'C'):
-    case GST_MAKE_FOURCC ('t', 's', 'c', 'c'):
-      caps = gst_caps_new_simple ("video/x-camtasia", NULL);
+    case GST_MAKE_FOURCC ('t', 's', 'c', 'c'):{
+      if (strf) {
+        gint depth = (strf->bit_cnt != 0) ? (gint) strf->bit_cnt : 24;
+
+        caps = gst_caps_new_simple ("video/x-camtasia", "depth", G_TYPE_INT,
+            depth, NULL);
+      } else {
+        /* template caps */
+        caps = gst_caps_new_simple ("video/x-camtasia", NULL);
+      }
       if (codec_name)
         *codec_name = g_strdup ("TechSmith Camtasia");
       break;
+    }
 
     case GST_MAKE_FOURCC ('V', 'C', 'R', '1'):
       caps = gst_caps_new_simple ("video/x-ati-vcr",
@@ -557,6 +587,12 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("VMWare NC Video");
       break;
 
+    case GST_MAKE_FOURCC ('d', 'r', 'a', 'c'):
+      caps = gst_caps_new_simple ("video/x-dirac", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Dirac");
+      break;
+
     default:
       GST_WARNING ("Unknown video fourcc %" GST_FOURCC_FORMAT,
           GST_FOURCC_ARGS (codec_fcc));
@@ -605,12 +641,16 @@ gst_riff_create_video_caps (guint32 codec_fcc,
           GST_BUFFER_SIZE (palette));
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-      gint n;
-      guint32 *data = (guint32 *) GST_BUFFER_DATA (copy);
+      {
+        guint8 *data = GST_BUFFER_DATA (copy);
+        gint n;
 
-      /* own endianness */
-      for (n = 0; n < num_colors; n++)
-        data[n] = GUINT32_FROM_LE (data[n]);
+        /* own endianness */
+        for (n = 0; n < num_colors; n++) {
+          GST_WRITE_UINT32_BE (data, GST_READ_UINT32_LE (data));
+          data += sizeof (guint32);
+        }
+      }
 #endif
       gst_caps_set_simple (caps, "palette_data", GST_TYPE_BUFFER, copy, NULL);
       gst_buffer_unref (copy);
@@ -708,33 +748,35 @@ gst_riff_create_audio_caps (guint16 codec_id,
 {
   gboolean block_align = FALSE, rate_chan = TRUE;
   GstCaps *caps = NULL;
-  gint rate_min = 1000, rate_max = 96000;
+  gint rate_min = 8000, rate_max = 96000;
   gint channels_max = 2;
 
   switch (codec_id) {
-    case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
-      caps = gst_caps_new_simple ("audio/mpeg",
-          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 3, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("MPEG-1 layer 3");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
-      caps = gst_caps_new_simple ("audio/mpeg",
-          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 2, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("MPEG-1 layer 2");
-      break;
-
     case GST_RIFF_WAVE_FORMAT_PCM:     /* PCM */
+      channels_max = 8;
+
       if (strf != NULL) {
         gint ba = strf->blockalign;
         gint ch = strf->channels;
-        gint ws = strf->size;
+        gint wd = ba * 8 / ch;
+        gint ws;
 
-        caps = gst_caps_new_simple ("audio/x-raw-int", "endianness", G_TYPE_INT, G_LITTLE_ENDIAN, "channels", G_TYPE_INT, ch,   /* needed for _add_layout() */
-            "width", G_TYPE_INT, (int) (ba * 8 / ch),
-            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, ws != 8, NULL);
+        if (strf->size > 32) {
+          GST_WARNING ("invalid depth (%d) of pcm audio, overwriting.",
+              strf->size);
+          strf->size = 8 * (guint) ceil (wd / 8.0);
+        }
+        /* in riff, the depth is stored in the size field but it just means that
+         * the _least_ significant bits are cleared. We can therefore just play
+         * the sample as if it had a depth == width */
+        /* For reference, the actual depth is in strf->size */
+        ws = wd;
+
+        caps = gst_caps_new_simple ("audio/x-raw-int",
+            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+            "channels", G_TYPE_INT, ch,
+            "width", G_TYPE_INT, wd,
+            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
 
         /* Add default MS channel layout if we have more than 2 channels,
          * but the layout isn't specified like with WAVEEXT below. Not sure
@@ -751,6 +793,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
             case 6:
               channel_mask = 0x3f;
               break;
+            case 8:
+              channel_mask = 0xff;
+              break;
             default:
               GST_WARNING ("don't know default layout for %d channels", ch);
               channel_mask = 0;
@@ -769,8 +814,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
         caps = gst_caps_from_string ("audio/x-raw-int, "
             "endianness = (int) LITTLE_ENDIAN, "
             "signed = (boolean) { true, false }, "
-            "width = (int) { 8, 16, 24, 32 }, "
-            "depth = (int) { 8, 16, 24, 32 }");
+            "width = (int) { 8, 16, 24, 32 }, " "depth = (int) [ 1, 32 ]");
       }
       if (codec_name && strf)
         *codec_name = g_strdup_printf ("Uncompressed %d-bit PCM audio",
@@ -785,7 +829,127 @@ gst_riff_create_audio_caps (guint16 codec_id,
       block_align = TRUE;
       break;
 
+    case GST_RIFF_WAVE_FORMAT_IEEE_FLOAT:
+      channels_max = 8;
+
+      if (strf != NULL) {
+        gint ba = strf->blockalign;
+        gint ch = strf->channels;
+        gint wd = ba * 8 / ch;
+
+        caps = gst_caps_new_simple ("audio/x-raw-float",
+            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+            "channels", G_TYPE_INT, ch, "width", G_TYPE_INT, wd, NULL);
+
+        /* Add default MS channel layout if we have more than 2 channels,
+         * but the layout isn't specified like with WAVEEXT below. Not sure
+         * if this is right, but at least it makes sound output work at all
+         * in those cases. Somebody with a a 5.1 setup should double-check
+         * with chan-id.wav */
+        if (ch > 2) {
+          guint32 channel_mask;
+
+          switch (ch) {
+            case 4:
+              channel_mask = 0x33;
+              break;
+            case 6:
+              channel_mask = 0x3f;
+              break;
+            case 8:
+              channel_mask = 0xff;
+              break;
+            default:
+              GST_WARNING ("don't know default layout for %d channels", ch);
+              channel_mask = 0;
+              break;
+          }
+
+          if (channel_mask) {
+            GST_DEBUG ("using default channel layout for %d channels", ch);
+            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+              GST_WARNING ("failed to add channel layout");
+            }
+          }
+        }
+      } else {
+        /* FIXME: this is pretty useless - we need fixed caps */
+        caps = gst_caps_from_string ("audio/x-raw-float, "
+            "endianness = (int) LITTLE_ENDIAN, " "width = (int) { 32, 64 }");
+      }
+      if (codec_name && strf)
+        *codec_name = g_strdup_printf ("Uncompressed %d-bit IEEE float audio",
+            strf->size);
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_IBM_CVSD:
+      goto unknown;
+
+    case GST_RIFF_WAVE_FORMAT_ALAW:
+      if (strf != NULL) {
+        if (strf->size != 8) {
+          GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
+              strf->size);
+          strf->size = 8;
+          strf->av_bps = 8;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+        if (strf->av_bps == 0 || strf->blockalign == 0) {
+          GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
+              strf->av_bps, strf->blockalign);
+          strf->av_bps = strf->size;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+      }
+      rate_max = 48000;
+      caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("A-law audio");
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_WMS:
+      caps = gst_caps_new_simple ("audio/x-wms", NULL);
+      if (strf != NULL) {
+        gst_caps_set_simple (caps,
+            "bitrate", G_TYPE_INT, strf->av_bps * 8,
+            "width", G_TYPE_INT, strf->size,
+            "depth", G_TYPE_INT, strf->size, NULL);
+      } else {
+        gst_caps_set_simple (caps,
+            "bitrate", GST_TYPE_INT_RANGE, 0, G_MAXINT, NULL);
+      }
+      if (codec_name)
+        *codec_name = g_strdup ("Windows Media Audio Speech");
+      block_align = TRUE;
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_MULAW:
+      if (strf != NULL) {
+        if (strf->size != 8) {
+          GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
+              strf->size);
+          strf->size = 8;
+          strf->av_bps = 8;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+        if (strf->av_bps == 0 || strf->blockalign == 0) {
+          GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
+              strf->av_bps, strf->blockalign);
+          strf->av_bps = strf->size;
+          strf->blockalign = strf->av_bps * strf->channels;
+        }
+      }
+      rate_max = 48000;
+      caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Mu-law audio");
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_OKI_ADPCM:
+      goto unknown;
+
     case GST_RIFF_WAVE_FORMAT_DVI_ADPCM:
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/x-adpcm",
           "layout", G_TYPE_STRING, "dvi", NULL);
       if (codec_name)
@@ -793,42 +957,29 @@ gst_riff_create_audio_caps (guint16 codec_id,
       block_align = TRUE;
       break;
 
-    case GST_RIFF_WAVE_FORMAT_MULAW:
-      if (strf != NULL && strf->size != 8) {
-        GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
-            strf->size);
-        strf->size = 8;
-        strf->av_bps = 8;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      if (strf != NULL && (strf->av_bps == 0 || strf->blockalign == 0)) {
-        GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
-            strf->av_bps, strf->blockalign);
-        strf->av_bps = strf->size;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+    case GST_RIFF_WAVE_FORMAT_GSM610:
+    case GST_RIFF_WAVE_FORMAT_MSN:
+      caps = gst_caps_new_simple ("audio/ms-gsm", NULL);
       if (codec_name)
-        *codec_name = g_strdup ("Mu-law audio");
+        *codec_name = g_strdup ("MS GSM audio");
       break;
 
-    case GST_RIFF_WAVE_FORMAT_ALAW:
-      if (strf != NULL && strf->size != 8) {
-        GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
-            strf->size);
-        strf->size = 8;
-        strf->av_bps = 8;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      if (strf != NULL && (strf->av_bps == 0 || strf->blockalign == 0)) {
-        GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
-            strf->av_bps, strf->blockalign);
-        strf->av_bps = strf->size;
-        strf->blockalign = strf->av_bps * strf->channels;
-      }
-      caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+    case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
+      rate_min = 16000;
+      rate_max = 48000;
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 2, NULL);
       if (codec_name)
-        *codec_name = g_strdup ("A-law audio");
+        *codec_name = g_strdup ("MPEG-1 layer 2");
+      break;
+
+    case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
+      rate_min = 8000;
+      rate_max = 48000;
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 3, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("MPEG-1 layer 3");
       break;
 
     case GST_RIFF_WAVE_FORMAT_VORBIS1: /* ogg/vorbis mode 1 */
@@ -848,6 +999,23 @@ gst_riff_create_audio_caps (guint16 codec_id,
       if (codec_name)
         *codec_name = g_strdup ("AC-3 audio");
       break;
+    case GST_RIFF_WAVE_FORMAT_DTS:
+      channels_max = 6;
+      caps = gst_caps_new_simple ("audio/x-dts", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("DTS audio");
+      /* wavparse is not always able to specify rate/channels for DTS-in-wav */
+      rate_chan = FALSE;
+      break;
+    case GST_RIFF_WAVE_FORMAT_AAC:
+    {
+      channels_max = 8;
+      caps = gst_caps_new_simple ("audio/mpeg",
+          "mpegversion", G_TYPE_INT, 4, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("MPEG-4 AAC audio");
+      break;
+    }
     case GST_RIFF_WAVE_FORMAT_WMAV1:
     case GST_RIFF_WAVE_FORMAT_WMAV2:
     case GST_RIFF_WAVE_FORMAT_WMAV3:
@@ -855,7 +1023,6 @@ gst_riff_create_audio_caps (guint16 codec_id,
       gint version = (codec_id - GST_RIFF_WAVE_FORMAT_WMAV1) + 1;
 
       channels_max = 6;
-
       block_align = TRUE;
 
       caps = gst_caps_new_simple ("audio/x-wma",
@@ -874,6 +1041,11 @@ gst_riff_create_audio_caps (guint16 codec_id,
       }
       break;
     }
+    case GST_RIFF_WAVE_FORMAT_WMAV3_L:
+    {
+      /* WMA Version 9 Lossless */
+      goto unknown;
+    }
     case GST_RIFF_WAVE_FORMAT_SONY_ATRAC3:
       caps = gst_caps_new_simple ("audio/x-vnd.sony.atrac3", NULL);
       if (codec_name)
@@ -886,13 +1058,17 @@ gst_riff_create_audio_caps (guint16 codec_id,
       guint32 subformat_guid[4];
       const guint8 *data;
 
-      if (strf_data == NULL || GST_BUFFER_SIZE (strf_data) != 22) {
+      channels_max = 8;
+
+      /* should be at least 22 bytes */
+      if (strf_data == NULL || GST_BUFFER_SIZE (strf_data) < 22) {
         GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %d (expected: 22)",
             (strf_data) ? GST_BUFFER_SIZE (strf_data) : -1);
         return NULL;
       }
 
       data = GST_BUFFER_DATA (strf_data);
+
       valid_bits_per_sample = GST_READ_UINT16_LE (data);
       channel_mask = GST_READ_UINT32_LE (data + 2);
       subformat_guid[0] = GST_READ_UINT32_LE (data + 6);
@@ -911,21 +1087,47 @@ gst_riff_create_audio_caps (guint16 codec_id,
           GST_DEBUG ("PCM");
           if (strf != NULL) {
             gint ba = strf->blockalign;
-            gint ws = strf->size;
-            gint depth = ws;
+            gint wd = ba * 8 / strf->channels;
+            gint ws;
+            
+            /* in riff, the depth is stored in the size field but it just
+             * means that the _least_ significant bits are cleared. We can
+             * therefore just play the sample as if it had a depth == width */
+            ws = wd;
 
-            if (valid_bits_per_sample != 0)
-              depth = valid_bits_per_sample;
+            /* For reference, use this to get the actual depth:
+             * ws = strf->size;
+             * if (valid_bits_per_sample != 0)
+             *   ws = valid_bits_per_sample; */
 
             caps = gst_caps_new_simple ("audio/x-raw-int",
                 "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
                 "channels", G_TYPE_INT, strf->channels,
-                "width", G_TYPE_INT, (int) (ba * 8 / strf->channels),
-                "depth", G_TYPE_INT, depth,
+                "width", G_TYPE_INT, wd,
+                "depth", G_TYPE_INT, ws,
                 "rate", G_TYPE_INT, strf->rate,
-                "signed", G_TYPE_BOOLEAN, (depth > 8) ? TRUE : FALSE, NULL);
+                "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
 
-            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+            /* If channel_mask == 0 and channels <= 2, 4, 6 or 8 let's
+             * assume default layout as some wav files don't have the
+             * channel mask set */
+
+            if (channel_mask == 0) {
+              switch (strf->channels) {
+                case 4:
+                  channel_mask = 0x33;
+                  break;
+                case 6:
+                  channel_mask = 0x3f;
+                  break;
+                case 8:
+                  channel_mask = 0xff;
+                  break;
+              }
+            }
+
+            if ((channel_mask != 0 || strf->channels > 2) &&
+                !gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
               GST_WARNING ("failed to add channel layout");
               gst_caps_unref (caps);
               caps = NULL;
@@ -938,23 +1140,147 @@ gst_riff_create_audio_caps (guint16 codec_id,
             }
           }
         } else if (subformat_guid[0] == 0x00000003) {
-          GST_DEBUG ("FIXME: handle IEEE float format");
-        }
+          GST_DEBUG ("FLOAT");
+          if (strf != NULL) {
+            gint ba = strf->blockalign;
+            gint ws = strf->size;
+            gint wd = ba * 8 / strf->channels;
 
-        if (caps == NULL) {
-          GST_WARNING ("Unknown WAVE_FORMAT_EXTENSIBLE audio format");
-          return NULL;
+            if (valid_bits_per_sample != 0)
+              ws = valid_bits_per_sample;
+
+            caps = gst_caps_new_simple ("audio/x-raw-float",
+                "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+                "channels", G_TYPE_INT, strf->channels,
+                "width", G_TYPE_INT, wd, "rate", G_TYPE_INT, strf->rate, NULL);
+
+            /* If channel_mask == 0 and channels <= 2, 4, 6 or 8 let's
+             * assume default layout as some wav files don't have the
+             * channel mask set */
+
+            if (channel_mask == 0) {
+              switch (strf->channels) {
+                case 4:
+                  channel_mask = 0x33;
+                  break;
+                case 6:
+                  channel_mask = 0x3f;
+                  break;
+                case 8:
+                  channel_mask = 0xff;
+                  break;
+              }
+            }
+
+            if ((channel_mask != 0 || strf->channels > 2) &&
+                !gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+              GST_WARNING ("failed to add channel layout");
+              gst_caps_unref (caps);
+              caps = NULL;
+            }
+            rate_chan = FALSE;
+
+            if (codec_name) {
+              *codec_name =
+                  g_strdup_printf ("Uncompressed %d-bit IEEE float audio",
+                  strf->size);
+            }
+          }
+        } else if (subformat_guid[0] == 00000006) {
+          GST_DEBUG ("ALAW");
+          if (strf != NULL) {
+            if (strf->size != 8) {
+              GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
+                  strf->size);
+              strf->size = 8;
+              strf->av_bps = 8;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+            if (strf->av_bps == 0 || strf->blockalign == 0) {
+              GST_WARNING
+                  ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
+                  strf->av_bps, strf->blockalign);
+              strf->av_bps = strf->size;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+          }
+          rate_max = 48000;
+          caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+
+          if (codec_name)
+            *codec_name = g_strdup ("A-law audio");
+        } else if (subformat_guid[0] == 0x00000007) {
+          GST_DEBUG ("MULAW");
+          if (strf != NULL) {
+            if (strf->size != 8) {
+              GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
+                  strf->size);
+              strf->size = 8;
+              strf->av_bps = 8;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+            if (strf->av_bps == 0 || strf->blockalign == 0) {
+              GST_WARNING
+                  ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
+                  strf->av_bps, strf->blockalign);
+              strf->av_bps = strf->size;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+          }
+          rate_max = 48000;
+          caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+          if (codec_name)
+            *codec_name = g_strdup ("Mu-law audio");
+        } else if (subformat_guid[0] == 0x00000092) {
+          GST_DEBUG ("FIXME: handle DOLBY AC3 SPDIF format");
         }
+      } else if (subformat_guid[0] == 0x6ba47966 &&
+          subformat_guid[1] == 0x41783f83 &&
+          subformat_guid[2] == 0xf0006596 && subformat_guid[3] == 0xe59262bf) {
+        caps = gst_caps_new_simple ("application/x-ogg-avi", NULL);
+        if (codec_name)
+          *codec_name = g_strdup ("Ogg-AVI");
+      }
+
+      if (caps == NULL) {
+        GST_WARNING ("Unknown WAVE_FORMAT_EXTENSIBLE audio format");
+        return NULL;
       }
       break;
+    }
+      /* can anything decode these? pitfdll? */
+    case GST_RIFF_WAVE_FORMAT_VOXWARE:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_BYTE_ALIGNED:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_AC8:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_AC10:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_AC16:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_AC20:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_METAVOICE:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_METASOUND:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_RT29HW:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_VR12:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_VR18:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_TQ40:
+    case GST_RIFF_WAVE_FORMAT_VOXWARE_TQ60:{
+      caps = gst_caps_new_simple ("audio/x-voxware",
+          "voxwaretype", G_TYPE_INT, (gint) codec_id, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Voxware");
+      break;
     }
     default:
+    unknown:
       GST_WARNING ("Unknown audio tag 0x%04x", codec_id);
       return NULL;
   }
 
   if (strf != NULL) {
     if (rate_chan) {
+      if (strf->channels > channels_max)
+        goto too_many_channels;
+      if (strf->rate < rate_min || strf->rate > rate_max)
+        goto invalid_rate;
+
       gst_caps_set_simple (caps,
           "rate", G_TYPE_INT, strf->rate,
           "channels", G_TYPE_INT, strf->channels, NULL);
@@ -982,6 +1308,20 @@ gst_riff_create_audio_caps (guint16 codec_id,
   }
 
   return caps;
+
+  /* ERROR */
+too_many_channels:
+  GST_WARNING
+      ("Stream claims to contain %u channels, but format only supports %d",
+      strf->channels, channels_max);
+  gst_caps_unref (caps);
+  return NULL;
+invalid_rate:
+  GST_WARNING
+      ("Stream with sample_rate %u, but format only supports %d .. %d",
+      strf->rate, rate_min, rate_max);
+  gst_caps_unref (caps);
+  return NULL;
 }
 
 GstCaps *
@@ -1017,63 +1357,70 @@ gst_riff_create_iavs_caps (guint32 codec_fcc,
 GstCaps *
 gst_riff_create_video_template_caps (void)
 {
-  guint32 tags[] = {
-    GST_MAKE_FOURCC ('I', '4', '2', '0'),
-    GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'),
-    GST_MAKE_FOURCC ('Y', 'V', 'U', '9'),
-    GST_MAKE_FOURCC ('M', 'J', 'P', 'G'),
+  static const guint32 tags[] = {
+    GST_MAKE_FOURCC ('3', 'I', 'V', '1'),
+    GST_MAKE_FOURCC ('A', 'S', 'V', '1'),
+    GST_MAKE_FOURCC ('A', 'S', 'V', '2'),
+    GST_MAKE_FOURCC ('C', 'L', 'J', 'R'),
+    GST_MAKE_FOURCC ('C', 'Y', 'U', 'V'),
+    GST_MAKE_FOURCC ('D', 'I', 'B', ' '),
+    GST_MAKE_FOURCC ('D', 'I', 'V', '3'),
+    GST_MAKE_FOURCC ('D', 'I', 'V', 'X'),
+    GST_MAKE_FOURCC ('D', 'U', 'C', 'K'),
     GST_MAKE_FOURCC ('D', 'V', 'S', 'D'),
+    GST_MAKE_FOURCC ('D', 'X', '5', '0'),
+    GST_MAKE_FOURCC ('F', 'L', 'V', '1'),
+    GST_MAKE_FOURCC ('H', '2', '6', '3'),
+    GST_MAKE_FOURCC ('H', '2', '6', '4'),
+    GST_MAKE_FOURCC ('H', 'F', 'Y', 'U'),
+    GST_MAKE_FOURCC ('I', '2', '6', '3'),
+    GST_MAKE_FOURCC ('I', '4', '2', '0'),
+    GST_MAKE_FOURCC ('I', 'V', '3', '2'),
+    GST_MAKE_FOURCC ('I', 'V', '4', '1'),
+    GST_MAKE_FOURCC ('I', 'V', '5', '0'),
+    GST_MAKE_FOURCC ('L', '2', '6', '3'),
+    GST_MAKE_FOURCC ('M', '2', '6', '3'),
+    GST_MAKE_FOURCC ('M', '4', 'S', '2'),
+    GST_MAKE_FOURCC ('M', 'J', 'P', 'G'),
+    GST_MAKE_FOURCC ('M', 'P', '4', '2'),
+    GST_MAKE_FOURCC ('M', 'P', '4', '3'),
+    GST_MAKE_FOURCC ('M', 'P', 'E', 'G'),
+    GST_MAKE_FOURCC ('M', 'P', 'G', '2'),
+    GST_MAKE_FOURCC ('M', 'P', 'G', '4'),
+    GST_MAKE_FOURCC ('M', 'S', 'Z', 'H'),
+    GST_MAKE_FOURCC ('P', 'N', 'G', ' '),
+    GST_MAKE_FOURCC ('R', 'L', 'E', ' '),
+    GST_MAKE_FOURCC ('R', 'T', '2', '1'),
+    GST_MAKE_FOURCC ('S', 'P', '5', '3'),
+    GST_MAKE_FOURCC ('T', 'M', '2', '0'),
+    GST_MAKE_FOURCC ('T', 'S', 'C', 'C'),
+    GST_MAKE_FOURCC ('U', 'L', 'T', 'I'),
+    GST_MAKE_FOURCC ('V', 'C', 'R', '1'),
+    GST_MAKE_FOURCC ('V', 'C', 'R', '2'),
+    GST_MAKE_FOURCC ('V', 'D', 'O', 'W'),
+    GST_MAKE_FOURCC ('V', 'I', 'V', 'O'),
+    GST_MAKE_FOURCC ('V', 'M', 'n', 'c'),
+    GST_MAKE_FOURCC ('V', 'P', '3', ' '),
+    GST_MAKE_FOURCC ('V', 'S', 'S', 'H'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '1'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '2'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '3'),
-    GST_MAKE_FOURCC ('M', 'P', 'G', '4'),
-    GST_MAKE_FOURCC ('M', 'P', '4', '2'),
-    GST_MAKE_FOURCC ('M', 'P', '4', '3'),
-    GST_MAKE_FOURCC ('H', 'F', 'Y', 'U'),
-    GST_MAKE_FOURCC ('D', 'I', 'V', '3'),
-    GST_MAKE_FOURCC ('M', 'P', 'E', 'G'),
-    GST_MAKE_FOURCC ('H', '2', '6', '3'),
-    GST_MAKE_FOURCC ('I', '2', '6', '3'),
-    GST_MAKE_FOURCC ('h', '2', '6', '4'),
-    GST_MAKE_FOURCC ('D', 'I', 'V', 'X'),
-    GST_MAKE_FOURCC ('D', 'X', '5', '0'),
     GST_MAKE_FOURCC ('X', 'V', 'I', 'D'),
-    GST_MAKE_FOURCC ('3', 'I', 'V', '1'),
-    GST_MAKE_FOURCC ('c', 'v', 'i', 'd'),
-    GST_MAKE_FOURCC ('m', 's', 'v', 'c'),
-    GST_MAKE_FOURCC ('R', 'L', 'E', ' '),
-    GST_MAKE_FOURCC ('D', 'I', 'B', ' '),
     GST_MAKE_FOURCC ('X', 'x', 'a', 'n'),
-    GST_MAKE_FOURCC ('I', 'V', '3', '2'),
-    GST_MAKE_FOURCC ('I', 'V', '5', '0'),
-    GST_MAKE_FOURCC ('M', '4', 'S', '2'),
-    GST_MAKE_FOURCC ('M', 'S', 'Z', 'H'),
+    GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'),
+    GST_MAKE_FOURCC ('Y', 'V', 'U', '9'),
     GST_MAKE_FOURCC ('Z', 'L', 'I', 'B'),
-    GST_MAKE_FOURCC ('A', 'S', 'V', '1'),
-    GST_MAKE_FOURCC ('A', 'S', 'V', '2'),
-    GST_MAKE_FOURCC ('V', 'C', 'R', '1'),
-    GST_MAKE_FOURCC ('V', 'C', 'R', '2'),
-    GST_MAKE_FOURCC ('C', 'L', 'J', 'R'),
-    GST_MAKE_FOURCC ('I', 'V', '4', '1'),
-    GST_MAKE_FOURCC ('R', 'T', '2', '1'),
-    GST_MAKE_FOURCC ('D', 'U', 'C', 'K'),
-    GST_MAKE_FOURCC ('T', 'M', '2', '0'),
-    GST_MAKE_FOURCC ('U', 'L', 'T', 'I'),
-    GST_MAKE_FOURCC ('V', 'P', '3', ' '),
-    GST_MAKE_FOURCC ('T', 'S', 'C', 'C'),
-    GST_MAKE_FOURCC ('S', 'P', '5', '3'),
-    GST_MAKE_FOURCC ('P', 'N', 'G', ' '),
-    GST_MAKE_FOURCC ('C', 'Y', 'U', 'V'),
-    GST_MAKE_FOURCC ('F', 'L', 'V', '1'),
-    GST_MAKE_FOURCC ('V', 'M', 'n', 'c'),
-    /* FILL ME */
-    0
+    GST_MAKE_FOURCC ('c', 'v', 'i', 'd'),
+    GST_MAKE_FOURCC ('h', '2', '6', '4'),
+    GST_MAKE_FOURCC ('m', 's', 'v', 'c'),
+    GST_MAKE_FOURCC ('x', '2', '6', '3')
+        /* FILL ME */
   };
   guint i;
   GstCaps *caps, *one;
 
   caps = gst_caps_new_empty ();
-  for (i = 0; tags[i] != 0; i++) {
+  for (i = 0; i < G_N_ELEMENTS (tags); i++) {
     one = gst_riff_create_video_caps (tags[i], NULL, NULL, NULL, NULL, NULL);
     if (one)
       gst_caps_append (caps, one);
@@ -1085,32 +1432,39 @@ gst_riff_create_video_template_caps (void)
 GstCaps *
 gst_riff_create_audio_template_caps (void)
 {
-  guint16 tags[] = {
+  static const guint16 tags[] = {
+    GST_RIFF_WAVE_FORMAT_GSM610,
     GST_RIFF_WAVE_FORMAT_MPEGL3,
     GST_RIFF_WAVE_FORMAT_MPEGL12,
     GST_RIFF_WAVE_FORMAT_PCM,
     GST_RIFF_WAVE_FORMAT_VORBIS1,
     GST_RIFF_WAVE_FORMAT_A52,
+    GST_RIFF_WAVE_FORMAT_DTS,
+    GST_RIFF_WAVE_FORMAT_AAC,
     GST_RIFF_WAVE_FORMAT_ALAW,
     GST_RIFF_WAVE_FORMAT_MULAW,
+    GST_RIFF_WAVE_FORMAT_WMS,
     GST_RIFF_WAVE_FORMAT_ADPCM,
     GST_RIFF_WAVE_FORMAT_DVI_ADPCM,
     GST_RIFF_WAVE_FORMAT_WMAV1,
     GST_RIFF_WAVE_FORMAT_WMAV2,
     GST_RIFF_WAVE_FORMAT_WMAV3,
     GST_RIFF_WAVE_FORMAT_SONY_ATRAC3,
-    /* FILL ME */
-    0
+    GST_RIFF_WAVE_FORMAT_IEEE_FLOAT,
+    GST_RIFF_WAVE_FORMAT_VOXWARE
+        /* FILL ME */
   };
   guint i;
   GstCaps *caps, *one;
 
   caps = gst_caps_new_empty ();
-  for (i = 0; tags[i] != 0; i++) {
+  for (i = 0; i < G_N_ELEMENTS (tags); i++) {
     one = gst_riff_create_audio_caps (tags[i], NULL, NULL, NULL, NULL, NULL);
     if (one)
       gst_caps_append (caps, one);
   }
+  one = gst_caps_new_simple ("application/x-ogg-avi", NULL);
+  gst_caps_append (caps, one);
 
   return caps;
 }
@@ -1118,16 +1472,15 @@ gst_riff_create_audio_template_caps (void)
 GstCaps *
 gst_riff_create_iavs_template_caps (void)
 {
-  guint32 tags[] = {
-    GST_MAKE_FOURCC ('D', 'V', 'S', 'D'),
-    /* FILL ME */
-    0
+  static const guint32 tags[] = {
+    GST_MAKE_FOURCC ('D', 'V', 'S', 'D')
+        /* FILL ME */
   };
   guint i;
   GstCaps *caps, *one;
 
   caps = gst_caps_new_empty ();
-  for (i = 0; tags[i] != 0; i++) {
+  for (i = 0; i < G_N_ELEMENTS (tags); i++) {
     one = gst_riff_create_iavs_caps (tags[i], NULL, NULL, NULL, NULL, NULL);
     if (one)
       gst_caps_append (caps, one);
