@@ -25,7 +25,6 @@
 
 #include <string.h>
 #include <gst/gstutils.h>
-#include <gst/tag/tag.h>
 
 #include "riff-read.h"
 
@@ -90,7 +89,7 @@ skip_junk:
     goto too_small;
 
   *_chunk_data = buf;
-  *_offset += 8 + GST_ROUND_UP_2 (size);
+  *_offset += 8 + ((size + 1) & ~1);
 
   return GST_FLOW_OK;
 
@@ -113,11 +112,11 @@ too_small:
  *          by the read size by this function.
  * @fourcc: fourcc (returned by this function0 of the chunk.
  * @chunk_data: buffer (returned by the function) containing the
- *              chunk data, which may be NULL if chunksize == 0
+ *              chunk data.
  *
  * Reads a single chunk.
  *
- * Returns: FALSE on error, TRUE otherwise
+ * Returns: the fourcc tag of this chunk, or FALSE on error
  */
 gboolean
 gst_riff_parse_chunk (GstElement * element, GstBuffer * buf,
@@ -153,10 +152,6 @@ gst_riff_parse_chunk (GstElement * element, GstBuffer * buf,
   GST_DEBUG_OBJECT (element, "fourcc=%" GST_FOURCC_FORMAT ", size=%u",
       GST_FOURCC_ARGS (fourcc), size);
 
-  /* be paranoid: size may be nonsensical value here, such as (guint) -1 */
-  if (G_UNLIKELY (size > G_MAXINT))
-    goto bogus_size;
-
   if (bufsize < size + 8 + offset) {
     GST_DEBUG_OBJECT (element,
         "Needed chunk data (%d) is more than available (%d), shortcutting",
@@ -170,7 +165,7 @@ gst_riff_parse_chunk (GstElement * element, GstBuffer * buf,
     *chunk_data = NULL;
 
   *_fourcc = fourcc;
-  *_offset += 8 + GST_ROUND_UP_2 (size);
+  *_offset += 8 + ((size + 1) & ~1);
 
   return TRUE;
 
@@ -187,11 +182,6 @@ too_small:
         offset, bufsize, 8);
     return FALSE;
   }
-bogus_size:
-  {
-    GST_ERROR_OBJECT (element, "Broken file: bogus chunk size %u", size);
-    return FALSE;
-  }
 }
 
 /**
@@ -226,7 +216,7 @@ gst_riff_parse_file_header (GstElement * element,
 
   data = GST_BUFFER_DATA (buf);
   tag = GST_READ_UINT32_LE (data);
-  if (tag != GST_RIFF_TAG_RIFF && tag != GST_RIFF_TAG_AVF0)
+  if (tag != GST_RIFF_TAG_RIFF)
     goto not_riff;
 
   *doctype = GST_READ_UINT32_LE (data + 8);
@@ -587,6 +577,78 @@ too_small:
   }
 }
 
+static gchar *
+freeform_string_to_utf8 (const gchar * data, gint size, const gchar ** env_vars)
+{
+  const gchar *env = NULL;
+  gsize bytes_read;
+  gchar *utf8 = NULL;
+
+  g_return_val_if_fail (data != NULL, NULL);
+
+  if (size < 0)
+    size = strlen (data);
+
+  /* Should we try the charsets specified
+   * via environment variables FIRST ? */
+  if (g_utf8_validate (data, size, NULL))
+    return g_strndup (data, size);
+
+  while ((env == NULL || *env == '\0') && env_vars && *env_vars != NULL) {
+    env = g_getenv (*env_vars);
+    ++env_vars;
+  }
+
+  /* Try charsets specified via the environment */
+  if (env != NULL && *env != '\0') {
+    gchar **c, **csets;
+
+    csets = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, -1);
+
+    for (c = csets; c && *c; ++c) {
+      if ((utf8 = g_convert (data, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
+        if (bytes_read == size) {
+          g_strfreev (csets);
+          goto beach;
+        }
+        g_free (utf8);
+        utf8 = NULL;
+      }
+    }
+
+    g_strfreev (csets);
+  }
+
+  /* Try current locale (if not UTF-8) */
+  if (!g_get_charset (&env)) {
+    if ((utf8 = g_locale_to_utf8 (data, size, &bytes_read, NULL, NULL))) {
+      if (bytes_read == size) {
+        goto beach;
+      }
+      g_free (utf8);
+      utf8 = NULL;
+    }
+  }
+
+  /* Try ISO-8859-1 */
+  utf8 = g_convert (data, size, "UTF-8", "ISO-8859-1", &bytes_read, NULL, NULL);
+  if (utf8 != NULL && bytes_read == size) {
+    goto beach;
+  }
+
+  g_free (utf8);
+  return NULL;
+
+beach:
+
+  g_strchomp (utf8);
+  if (utf8 && utf8[0] != '\0')
+    return utf8;
+
+  g_free (utf8);
+  return NULL;
+}
+
 /**
  * gst_riff_parse_info:
  * @element: caller element (used for debugging/error).
@@ -623,10 +685,6 @@ gst_riff_parse_info (GstElement * element,
     tsize = GST_READ_UINT32_LE (data + 4);
     size -= 8;
     data += 8;
-
-    GST_DEBUG ("tag %" GST_FOURCC_FORMAT ", size %u",
-        GST_FOURCC_ARGS (tag), tsize);
-
     if (tsize > size) {
       GST_WARNING_OBJECT (element,
           "Tagsize %d is larger than available data %d", tsize, size);
@@ -669,7 +727,7 @@ gst_riff_parse_info (GstElement * element,
         type = GST_TAG_GENRE;
         break;
       case GST_RIFF_INFO_IKEY:
-        type = GST_TAG_KEYWORDS;
+        type = NULL; /*"Keywords"; */ ;
         break;
       case GST_RIFF_INFO_ILGT:
         type = NULL;            /*"Lightness"; */
@@ -718,7 +776,7 @@ gst_riff_parse_info (GstElement * element,
       };
       gchar *val;
 
-      val = gst_tag_freeform_string_to_utf8 ((gchar *) data, tsize, env_vars);
+      val = freeform_string_to_utf8 ((gchar *) data, tsize, env_vars);
 
       if (val) {
         gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, type, val, NULL);
@@ -728,12 +786,6 @@ gst_riff_parse_info (GstElement * element,
       }
     }
 
-    if (tsize & 1) {
-      tsize++;
-      if (tsize > size)
-        tsize = size;
-    }
-
     data += tsize;
     size -= tsize;
   }
