@@ -12,6 +12,17 @@
  * Library General Public License for more 
  */
 
+/**
+ * SECTION:gstbasertppayload
+ * @short_description: Base class for RTP payloader
+ *
+ * <refsect2>
+ * <para>
+ * Provides a base class for RTP payloaders
+ * </para>
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -22,7 +33,7 @@
 
 #include "gstbasertppayload.h"
 
-GST_DEBUG_CATEGORY (basertppayload_debug);
+GST_DEBUG_CATEGORY_STATIC (basertppayload_debug);
 #define GST_CAT_DEFAULT (basertppayload_debug)
 
 /* BaseRTPPayload signals and args */
@@ -32,12 +43,19 @@ enum
   LAST_SIGNAL
 };
 
+/* FIXME 0.11, a better default is the Ethernet MTU of 
+ * 1500 - sizeof(headers) as pointed out by marcelm in IRC:
+ * So an Ethernet MTU of 1500, minus 60 for the max IP, minus 8 for UDP, gives
+ * 1432 bytes or so.  And that should be adjusted downward further for other
+ * encapsulations like PPPoE, so 1400 at most.
+ */
 #define DEFAULT_MTU                     1024
 #define DEFAULT_PT                      96
 #define DEFAULT_SSRC                    -1
 #define DEFAULT_TIMESTAMP_OFFSET        -1
 #define DEFAULT_SEQNUM_OFFSET           -1
 #define DEFAULT_MAX_PTIME               -1
+#define DEFAULT_MIN_PTIME               0
 
 enum
 {
@@ -48,6 +66,7 @@ enum
   PROP_TIMESTAMP_OFFSET,
   PROP_SEQNUM_OFFSET,
   PROP_MAX_PTIME,
+  PROP_MIN_PTIME,
   PROP_TIMESTAMP,
   PROP_SEQNUM
 };
@@ -73,6 +92,8 @@ static GstStateChangeReturn gst_basertppayload_change_state (GstElement *
 
 static GstElementClass *parent_class = NULL;
 
+/* FIXME 0.11: API should be changed to gst_base_typ_payload_xyz */
+
 GType
 gst_basertppayload_get_type (void)
 {
@@ -112,7 +133,7 @@ gst_basertppayload_class_init (GstBaseRTPPayloadClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = gst_basertppayload_finalize;
 
@@ -144,6 +165,17 @@ gst_basertppayload_class_init (GstBaseRTPPayloadClass * klass)
       g_param_spec_int64 ("max-ptime", "Max packet time",
           "Maximum duration of the packet data in ns (-1 = unlimited up to MTU)",
           -1, G_MAXINT64, DEFAULT_MAX_PTIME, G_PARAM_READWRITE));
+  /**
+   * GstBaseRTPAudioPayload:min-ptime:
+   *
+   * Minimum duration of the packet data in ns (can't go above MTU)
+   *
+   * Since: 0.10.13
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MIN_PTIME,
+      g_param_spec_int64 ("min-ptime", "Min packet time",
+          "Minimum duration of the packet data in ns (can't go above MTU)",
+          0, G_MAXINT64, DEFAULT_MIN_PTIME, G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TIMESTAMP,
       g_param_spec_uint ("timestamp", "Timestamp",
@@ -195,6 +227,10 @@ gst_basertppayload_init (GstBaseRTPPayload * basertppayload, gpointer g_class)
   basertppayload->ssrc = DEFAULT_SSRC;
   basertppayload->ts_offset = DEFAULT_TIMESTAMP_OFFSET;
   basertppayload->max_ptime = DEFAULT_MAX_PTIME;
+  basertppayload->min_ptime = DEFAULT_MIN_PTIME;
+
+  basertppayload->media = NULL;
+  basertppayload->encoding_name = NULL;
 
   basertppayload->clock_rate = 0;
 }
@@ -213,6 +249,11 @@ gst_basertppayload_finalize (GObject * object)
   g_rand_free (basertppayload->ts_rand);
   basertppayload->ts_rand = NULL;
 
+  g_free (basertppayload->media);
+  basertppayload->media = NULL;
+  g_free (basertppayload->encoding_name);
+  basertppayload->encoding_name = NULL;
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -239,9 +280,20 @@ static gboolean
 gst_basertppayload_event (GstPad * pad, GstEvent * event)
 {
   GstBaseRTPPayload *basertppayload;
+  GstBaseRTPPayloadClass *basertppayload_class;
   gboolean res;
 
   basertppayload = GST_BASE_RTP_PAYLOAD (gst_pad_get_parent (pad));
+  basertppayload_class = GST_BASE_RTP_PAYLOAD_GET_CLASS (basertppayload);
+
+  if (basertppayload_class->handle_event) {
+    res = basertppayload_class->handle_event (pad, event);
+    if (res) {
+      gst_object_unref (basertppayload);
+
+      return res;
+    }
+  }
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
@@ -381,6 +433,7 @@ gst_basertppayload_push (GstBaseRTPPayload * payload, GstBuffer * buffer)
   /* update first, so that the property is set to the last
    * seqnum pushed */
   payload->seqnum++;
+  GST_LOG_OBJECT (payload, "setting RTP seqnum %d", payload->seqnum);
   gst_rtp_buffer_set_seq (buffer, payload->seqnum);
 
   /* add our random offset to the timestamp */
@@ -397,6 +450,7 @@ gst_basertppayload_push (GstBaseRTPPayload * payload, GstBuffer * buffer)
 
     ts += rtime;
   }
+  GST_LOG_OBJECT (payload, "setting RTP timestamp %u", (guint) ts);
   gst_rtp_buffer_set_timestamp (buffer, ts);
 
   payload->timestamp = ts;
@@ -448,6 +502,9 @@ gst_basertppayload_set_property (GObject * object, guint prop_id,
     case PROP_MAX_PTIME:
       basertppayload->max_ptime = g_value_get_int64 (value);
       break;
+    case PROP_MIN_PTIME:
+      basertppayload->min_ptime = g_value_get_int64 (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -481,6 +538,9 @@ gst_basertppayload_get_property (GObject * object, guint prop_id,
     case PROP_MAX_PTIME:
       g_value_set_int64 (value, basertppayload->max_ptime);
       break;
+    case PROP_MIN_PTIME:
+      g_value_set_int64 (value, basertppayload->min_ptime);
+      break;
     case PROP_TIMESTAMP:
       g_value_set_uint (value, basertppayload->timestamp);
       break;
