@@ -1,8 +1,6 @@
 /* GStreamer
  * Copyright (C) 2005 Stefan Kost <ensonic@users.sf.net>
  *
- * gstaudiotestsrc.c:
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
@@ -22,8 +20,10 @@
  * SECTION:element-audiotestsrc
  *
  * <refsect2>
+ * <para>
  * AudioTestSrc can be used to generate basic audio signals. It support several
  * different waveforms and allows you to set the base frequency and volume.
+ * </para>
  * <title>Example launch line</title>
  * <para>
  * <programlisting>
@@ -33,7 +33,7 @@
  * </para>
  * <para>
  * <programlisting>
- * gst-launch audiotestsrc wave=2 freq=200 ! audioconvert ! tee name=t ! alsasink t. ! libvisual_lv_scope ! ffmpegcolorspace ! xvimagesink
+ * gst-launch audiotestsrc wave=2 freq=200 ! audioconvert ! tee name=t ! queue ! alsasink t. ! queue ! libvisual_lv_scope ! ffmpegcolorspace ! xvimagesink
  * </programlisting>
  * In this example a saw wave is generated. The wave is shown using a
  * scope visualizer from libvisual, allowing you to visually verify that
@@ -64,12 +64,14 @@
 
 #define M_PI_M2 ( M_PI + M_PI )
 
-GstElementDetails gst_audio_test_src_details = {
-  "Audio test source",
-  "Source/Audio",
-  "Creates audio test signals of given frequency and volume",
-  "Stefan Kost <ensonic@users.sf.net>"
-};
+GST_DEBUG_CATEGORY_STATIC (audio_test_src_debug);
+#define GST_CAT_DEFAULT audio_test_src_debug
+
+static const GstElementDetails gst_audio_test_src_details =
+GST_ELEMENT_DETAILS ("Audio test source",
+    "Source/Audio",
+    "Creates audio test signals of given frequency and volume",
+    "Stefan Kost <ensonic@users.sf.net>");
 
 
 enum
@@ -104,7 +106,7 @@ static GType
 gst_audiostestsrc_wave_get_type (void)
 {
   static GType audiostestsrc_wave_type = 0;
-  static GEnumValue audiostestsrc_waves[] = {
+  static const GEnumValue audiostestsrc_waves[] = {
     {GST_AUDIO_TEST_SRC_WAVE_SINE, "Sine", "sine"},
     {GST_AUDIO_TEST_SRC_WAVE_SQUARE, "Square", "square"},
     {GST_AUDIO_TEST_SRC_WAVE_SAW, "Saw", "saw"},
@@ -116,7 +118,7 @@ gst_audiostestsrc_wave_get_type (void)
     {0, NULL, NULL},
   };
 
-  if (!audiostestsrc_wave_type) {
+  if (G_UNLIKELY (audiostestsrc_wave_type == 0)) {
     audiostestsrc_wave_type = g_enum_register_static ("GstAudioTestSrcWave",
         audiostestsrc_waves);
   }
@@ -408,7 +410,7 @@ gst_audio_test_src_create_white_noise (GstAudioTestSrc * src, gint16 * samples)
   }
 }
 
-/* pink noise calculation is based on 
+/* pink noise calculation is based on
  * http://www.firstpr.com.au/dsp/pink-noise/phil_burk_19990905_patest_pink.c
  * which has been released under public domain
  * Many thanks Phil!
@@ -605,7 +607,8 @@ gst_audio_test_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
   GstAudioTestSrc *src = GST_AUDIO_TEST_SRC (basesrc);
   GstClockTime time;
 
-  time = segment->time = segment->start;
+  segment->time = segment->start;
+  time = segment->last_stop;
 
   /* now move to the time indicated */
   src->n_samples = time * src->samplerate / GST_SECOND;
@@ -636,6 +639,7 @@ static GstFlowReturn
 gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     guint length, GstBuffer ** buffer)
 {
+  GstFlowReturn res;
   GstAudioTestSrc *src;
   GstBuffer *buf;
   GstClockTime next_time;
@@ -646,6 +650,7 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
   if (src->eos_reached)
     return GST_FLOW_UNEXPECTED;
 
+  /* example for tagging generated data */
   if (!src->tags_pushed) {
     GstTagList *taglist;
     GstEvent *event;
@@ -660,6 +665,7 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     src->tags_pushed = TRUE;
   }
 
+  /* check for eos */
   if (src->check_seek_stop &&
       (src->n_samples_stop > src->n_samples) &&
       (src->n_samples_stop < src->n_samples + src->samples_per_buffer)
@@ -673,15 +679,17 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     src->generate_samples_per_buffer = src->samples_per_buffer;
     n_samples = src->n_samples + src->samples_per_buffer;
   }
-  next_time = n_samples * GST_SECOND / src->samplerate;
+  next_time = gst_util_uint64_scale (n_samples, GST_SECOND,
+      (guint64) src->samplerate);
 
-  buf =
-      gst_buffer_new_and_alloc (src->generate_samples_per_buffer *
-      sizeof (gint16));
-  gst_buffer_set_caps (buf, GST_PAD_CAPS (basesrc->srcpad));
+  /* allocate a new buffer suitable for this pad */
+  if ((res = gst_pad_alloc_buffer (basesrc->srcpad, src->n_samples,
+              src->generate_samples_per_buffer * sizeof (gint16),
+              GST_PAD_CAPS (basesrc->srcpad), &buf)) != GST_FLOW_OK) {
+    return res;
+  }
 
   GST_BUFFER_TIMESTAMP (buf) = src->timestamp_offset + src->running_time;
-  GST_BUFFER_OFFSET (buf) = src->n_samples;
   GST_BUFFER_OFFSET_END (buf) = n_samples;
   GST_BUFFER_DURATION (buf) = next_time - src->running_time;
 
@@ -767,6 +775,9 @@ plugin_init (GstPlugin * plugin)
   /* initialize gst controller library */
   gst_controller_init (NULL, NULL);
 
+  GST_DEBUG_CATEGORY_INIT (audio_test_src_debug, "audiotestsrc", 0,
+      "Audio Test Source");
+
   return gst_element_register (plugin, "audiotestsrc",
       GST_RANK_NONE, GST_TYPE_AUDIO_TEST_SRC);
 }
