@@ -167,12 +167,6 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_YUV420P:
       fmt = GST_MAKE_FOURCC ('I', '4', '2', '0');
       break;
-    case PIX_FMT_NV12:
-      fmt = GST_MAKE_FOURCC ('N', 'V', '1', '2');
-      break;
-    case PIX_FMT_NV21:
-      fmt = GST_MAKE_FOURCC ('N', 'V', '2', '1');
-      break;
     case PIX_FMT_YVU420P:
       fmt = GST_MAKE_FOURCC ('Y', 'V', '1', '2');
       break;
@@ -182,12 +176,6 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_UYVY422:
       fmt = GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y');
       break;
-    case PIX_FMT_YVYU422:
-      fmt = GST_MAKE_FOURCC ('Y', 'V', 'Y', 'U');
-      break;
-    case PIX_FMT_UYVY411:
-      fmt = GST_MAKE_FOURCC ('I', 'Y', 'U', '1');
-      break;
     case PIX_FMT_RGB24:
       bpp = depth = 24;
       endianness = G_BIG_ENDIAN;
@@ -294,38 +282,6 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       g_mask = 0x0000ff00;
       b_mask = 0x000000ff;
       a_mask = 0xff000000;
-#endif
-      break;
-    case PIX_FMT_ARGB32:
-      bpp = 32;
-      depth = 32;
-      endianness = G_BIG_ENDIAN;
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-      r_mask = 0xff000000;
-      g_mask = 0x00ff0000;
-      b_mask = 0x0000ff00;
-      a_mask = 0x000000ff;
-#else
-      r_mask = 0x000000ff;
-      g_mask = 0x0000ff00;
-      b_mask = 0x00ff0000;
-      a_mask = 0xff000000;
-#endif
-      break;
-    case PIX_FMT_ABGR32:
-      bpp = 32;
-      depth = 32;
-      endianness = G_BIG_ENDIAN;
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-      r_mask = 0x000000ff;
-      g_mask = 0x0000ff00;
-      b_mask = 0x00ff0000;
-      a_mask = 0xff000000;
-#else
-      r_mask = 0xff000000;
-      g_mask = 0x00ff0000;
-      b_mask = 0x0000ff00;
-      a_mask = 0x000000ff;
 #endif
       break;
     case PIX_FMT_YUV410P:
@@ -356,35 +312,13 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       bpp = depth = 8;
       endianness = G_BYTE_ORDER;
       break;
-    case PIX_FMT_V308:
-      fmt = GST_MAKE_FOURCC ('v', '3', '0', '8');
-      break;
     case PIX_FMT_AYUV4444:
       fmt = GST_MAKE_FOURCC ('A', 'Y', 'U', 'V');
       break;
-    case PIX_FMT_GRAY8:{
-      GstCaps *tmp;
-
+    case PIX_FMT_GRAY8:
       bpp = depth = 8;
       caps = gst_ff_vid_caps_new (context, "video/x-raw-gray",
           "bpp", G_TYPE_INT, bpp, "depth", G_TYPE_INT, depth, NULL);
-      tmp = gst_ff_vid_caps_new (context, "video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('Y', '8', '0', '0'),
-          NULL);
-      gst_caps_append (caps, tmp);
-    }
-      break;
-    case PIX_FMT_GRAY16_L:
-      bpp = depth = 16;
-      caps = gst_ff_vid_caps_new (context, "video/x-raw-gray",
-          "bpp", G_TYPE_INT, bpp, "depth", G_TYPE_INT, depth,
-          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN, NULL);
-      break;
-    case PIX_FMT_GRAY16_B:
-      bpp = depth = 16;
-      caps = gst_ff_vid_caps_new (context, "video/x-raw-gray",
-          "bpp", G_TYPE_INT, bpp, "depth", G_TYPE_INT, depth,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN, NULL);
       break;
     default:
       /* give up ... */
@@ -415,7 +349,7 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
             "bpp", G_TYPE_INT, bpp,
             "depth", G_TYPE_INT, depth,
             "endianness", G_TYPE_INT, endianness, NULL);
-        if (context && context->pix_fmt == PIX_FMT_PAL8) {
+        if (context) {
           gst_ffmpeg_set_palette (caps, context);
         }
       }
@@ -426,7 +360,10 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
   }
 
   if (caps != NULL) {
-    GST_DEBUG ("caps for pix_fmt=%d: %" GST_PTR_FORMAT, pix_fmt, caps);
+    char *str = gst_caps_to_string (caps);
+
+    GST_DEBUG ("caps for pix_fmt=%d: %s", pix_fmt, str);
+    g_free (str);
   } else {
     GST_LOG ("No caps found for pix_fmt=%d", pix_fmt);
   }
@@ -469,7 +406,10 @@ gst_ffmpeg_smpfmt_to_caps (enum SampleFormat sample_fmt,
   }
 
   if (caps != NULL) {
-    GST_DEBUG ("caps for sample_fmt=%d: %" GST_PTR_FORMAT, sample_fmt, caps);
+    char *str = gst_caps_to_string (caps);
+
+    GST_DEBUG ("caps for sample_fmt=%d: %s", sample_fmt, str);
+    g_free (str);
   } else {
     GST_LOG ("No caps found for sample_fmt=%d", sample_fmt);
   }
@@ -612,21 +552,9 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         case GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y'):
           context->pix_fmt = PIX_FMT_UYVY422;
           break;
-        case GST_MAKE_FOURCC ('Y', 'V', 'Y', 'U'):
-          context->pix_fmt = PIX_FMT_YVYU422;
-          break;
-        case GST_MAKE_FOURCC ('I', 'Y', 'U', '1'):
-          context->pix_fmt = PIX_FMT_UYVY411;
-          break;
         case GST_MAKE_FOURCC ('I', '4', '2', '0'):
           context->pix_fmt = PIX_FMT_YUV420P;
           break;
-        case GST_MAKE_FOURCC ('N', 'V', '1', '2'):
-          context->pix_fmt = PIX_FMT_NV12;
-          break;
-        case GST_MAKE_FOURCC ('N', 'V', '2', '1'):
-          context->pix_fmt = PIX_FMT_NV21;
-          break;
         case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
           context->pix_fmt = PIX_FMT_YVU420P;
           break;
@@ -642,18 +570,12 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         case GST_MAKE_FOURCC ('Y', 'V', 'U', '9'):
           context->pix_fmt = PIX_FMT_YVU410P;
           break;
-        case GST_MAKE_FOURCC ('v', '3', '0', '8'):
-          context->pix_fmt = PIX_FMT_V308;
-          break;
         case GST_MAKE_FOURCC ('A', 'Y', 'U', 'V'):
           context->pix_fmt = PIX_FMT_AYUV4444;
           break;
         case GST_MAKE_FOURCC ('Y', '4', '4', '4'):
           context->pix_fmt = PIX_FMT_YUV444P;
           break;
-        case GST_MAKE_FOURCC ('Y', '8', '0', '0'):
-          context->pix_fmt = PIX_FMT_GRAY8;
-          break;
       }
     }
   } else if (gst_structure_has_name (structure, "video/x-raw-rgb")) {
@@ -667,43 +589,40 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
             if (gst_structure_get_int (structure, "alpha_mask", &amask)) {
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
               if (rmask == 0x0000ff00)
-                context->pix_fmt = PIX_FMT_BGRA32;
-              else if (rmask == 0x00ff0000)
-                context->pix_fmt = PIX_FMT_RGBA32;
-              else if (rmask == 0xff000000)
-                context->pix_fmt = PIX_FMT_ARGB32;
-              else              // if (r_mask = 0x000000ff)
-                context->pix_fmt = PIX_FMT_ABGR32;
 #else
               if (rmask == 0x00ff0000)
-                context->pix_fmt = PIX_FMT_BGRA32;
-              else if (rmask == 0x0000ff00)
-                context->pix_fmt = PIX_FMT_RGBA32;
-              else if (rmask == 0x000000ff)
-                context->pix_fmt = PIX_FMT_ARGB32;
-              else              // if (rmask == 0xff000000)
-                context->pix_fmt = PIX_FMT_ABGR32;
 #endif
+                context->pix_fmt = PIX_FMT_BGRA32;
+              else
+                context->pix_fmt = PIX_FMT_RGBA32;
             } else {
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
               if (rmask == 0x00ff0000)
-                context->pix_fmt = PIX_FMT_RGB32;
-              else if (rmask == 0x0000ff00)
-                context->pix_fmt = PIX_FMT_BGR32;
-              else if (rmask == 0xff000000)
-                context->pix_fmt = PIX_FMT_xRGB32;
-              else              // if (rmask == 0x000000ff)
-                context->pix_fmt = PIX_FMT_BGRx32;
 #else
               if (rmask == 0x0000ff00)
-                context->pix_fmt = PIX_FMT_RGB32;
-              else if (rmask == 0x00ff0000)
-                context->pix_fmt = PIX_FMT_BGR32;
-              else if (rmask == 0x000000ff)
-                context->pix_fmt = PIX_FMT_xRGB32;
-              else              // if (rmask == 0xff000000)
-                context->pix_fmt = PIX_FMT_BGRx32;
 #endif
+                context->pix_fmt = PIX_FMT_RGB32;
+
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+              if (rmask == 0x0000ff00)
+#else
+              if (rmask == 0x00ff0000)
+#endif
+                context->pix_fmt = PIX_FMT_BGR32;
+
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+              if (rmask == 0xff000000)
+#else
+              if (rmask == 0x000000ff)
+#endif
+                context->pix_fmt = PIX_FMT_xRGB32;
+
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+              if (rmask == 0x000000ff)
+#else
+              if (rmask == 0xff000000)
+#endif
+                context->pix_fmt = PIX_FMT_BGRx32;
             }
             break;
           case 24:
@@ -744,17 +663,6 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         case 8:
           context->pix_fmt = PIX_FMT_GRAY8;
           break;
-        case 16:{
-          gint endianness = 0;
-
-          if (gst_structure_get_int (structure, "endianness", &endianness)) {
-            if (endianness == G_LITTLE_ENDIAN)
-              context->pix_fmt = PIX_FMT_GRAY16_L;
-            else if (endianness == G_BIG_ENDIAN)
-              context->pix_fmt = PIX_FMT_GRAY16_B;
-          }
-        }
-          break;
       }
     }
   }
@@ -801,8 +709,7 @@ gst_ffmpegcsp_caps_with_codectype (enum CodecType type,
  */
 int
 gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
-    uint8_t * ptr, enum PixelFormat pix_fmt, int width, int height,
-    int interlaced)
+    uint8_t * ptr, enum PixelFormat pix_fmt, int width, int height)
 {
   int size, w2, h2, size2;
   int stride, stride2;
@@ -810,8 +717,6 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
 
   pinfo = get_pix_fmt_info (pix_fmt);
 
-  picture->interlaced = interlaced;
-
   switch (pix_fmt) {
     case PIX_FMT_YUV420P:
     case PIX_FMT_YUV422P:
@@ -853,22 +758,6 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       picture->linesize[1] = stride2;
       picture->linesize[2] = stride2;
       return size + 2 * size2;
-    case PIX_FMT_NV12:
-    case PIX_FMT_NV21:
-      stride = GST_ROUND_UP_4 (width);
-      h2 = ROUND_UP_X (height, pinfo->y_chroma_shift);
-      size = stride * h2;
-      w2 = 2 * DIV_ROUND_UP_X (width, pinfo->x_chroma_shift);
-      stride2 = GST_ROUND_UP_4 (w2);
-      h2 = DIV_ROUND_UP_X (height, pinfo->y_chroma_shift);
-      size2 = stride2 * h2;
-      picture->data[0] = ptr;
-      picture->data[1] = picture->data[0] + size;
-      picture->data[2] = NULL;
-      picture->linesize[0] = stride;
-      picture->linesize[1] = stride2;
-      picture->linesize[2] = 0;
-      return size + size2;
     case PIX_FMT_RGB24:
     case PIX_FMT_BGR24:
       stride = GST_ROUND_UP_4 (width * 3);
@@ -881,10 +770,8 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
     case PIX_FMT_AYUV4444:
     case PIX_FMT_RGB32:
     case PIX_FMT_RGBA32:
-    case PIX_FMT_ARGB32:
     case PIX_FMT_BGR32:
     case PIX_FMT_BGRA32:
-    case PIX_FMT_ABGR32:
     case PIX_FMT_xRGB32:
     case PIX_FMT_BGRx32:
       stride = width * 4;
@@ -898,7 +785,6 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
     case PIX_FMT_RGB565:
     case PIX_FMT_YUV422:
     case PIX_FMT_UYVY422:
-    case PIX_FMT_YVYU422:
       stride = GST_ROUND_UP_4 (width * 2);
       size = stride * height;
       picture->data[0] = ptr;
@@ -906,14 +792,6 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       picture->data[2] = NULL;
       picture->linesize[0] = stride;
       return size;
-    case PIX_FMT_V308:
-      stride = GST_ROUND_UP_4 (width * 3);
-      size = stride * height;
-      picture->data[0] = ptr;
-      picture->data[1] = NULL;
-      picture->data[2] = NULL;
-      picture->linesize[0] = stride;
-      return size;
     case PIX_FMT_UYVY411:
       /* FIXME, probably not the right stride */
       stride = GST_ROUND_UP_4 (width);
@@ -931,15 +809,6 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       picture->data[2] = NULL;
       picture->linesize[0] = stride;
       return size;
-    case PIX_FMT_GRAY16_L:
-    case PIX_FMT_GRAY16_B:
-      stride = GST_ROUND_UP_4 (width * 2);
-      size = stride * height;
-      picture->data[0] = ptr;
-      picture->data[1] = NULL;
-      picture->data[2] = NULL;
-      picture->linesize[0] = stride;
-      return size;
     case PIX_FMT_MONOWHITE:
     case PIX_FMT_MONOBLACK:
       stride = GST_ROUND_UP_4 ((width + 7) >> 3);
