@@ -23,6 +23,9 @@
 
 #include <gst/gst.h>
 
+GST_DEBUG_CATEGORY_EXTERN (sub_parse_debug);
+#define GST_CAT_DEFAULT sub_parse_debug
+
 G_BEGIN_DECLS
 
 #define GST_TYPE_SUBPARSE \
@@ -30,10 +33,10 @@ G_BEGIN_DECLS
 #define GST_SUBPARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_SUBPARSE, GstSubParse))
 #define GST_SUBPARSE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_SUBPARSE, GstSubParse))
+  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_SUBPARSE, GstSubParseClass))
 #define GST_IS_SUBPARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_SUBPARSE))
-#define GST_IS_SUBPARSE_CLASS(obj) \
+#define GST_IS_SUBPARSE_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_SUBPARSE))
 
 typedef struct _GstSubParse GstSubParse;
@@ -45,7 +48,10 @@ typedef enum
   GST_SUB_PARSE_FORMAT_UNKNOWN = 0,
   GST_SUB_PARSE_FORMAT_MDVDSUB = 1,
   GST_SUB_PARSE_FORMAT_SUBRIP = 2,
-  GST_SUB_PARSE_FORMAT_MPSUB = 3
+  GST_SUB_PARSE_FORMAT_MPSUB = 3,
+  GST_SUB_PARSE_FORMAT_SAMI = 4,
+  GST_SUB_PARSE_FORMAT_TMPLAYER = 5,
+  GST_SUB_PARSE_FORMAT_MPL2 = 6
 } GstSubParseFormat;
 
 typedef struct {
@@ -53,6 +59,9 @@ typedef struct {
   GString *buf;
   guint64  start_time;
   guint64  duration;
+  GstSegment *segment;
+  gpointer user_data;
+  gdouble  fps;          /* used by microdvd parser */
 } ParserState;
 
 typedef gchar* (*Parser) (ParserState *state, const gchar *line);
@@ -73,6 +82,15 @@ struct _GstSubParse {
   /* seek */
   guint64 offset;
   guint64 next_offset;
+  
+  /* Segment */
+  GstSegment    segment;
+  GstSeekFlags  segment_flags;
+  gboolean      need_segment;
+  
+  gboolean flushing;
+  gboolean valid_utf8;
+  gchar   *encoding;
 };
 
 struct _GstSubParseClass {
