@@ -17,59 +17,78 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-videorate
+ * @short_description: retimestamps and drops/duplicates video frames to
+ *  match the source pad's framerate and create a perfect stream
+ *
+ * <refsect2>
+ * <para>
+ * This element takes an incoming stream of timestamped video frames.
+ * It will produce a perfect stream that matches the source pad's framerate.
+ *
+ * The correction is performed by dropping and duplicating frames, no fancy
+ * algorithm is used to interpolate frames (yet).
+ * </para>
+ * <para>
+ * By default the element will simply negotiate the same framerate on its
+ * source and sink pad.
+ * </para>
+ * <para>
+ * This operation is useful to link to elements that require a perfect stream.
+ * Typical examples are formats that do not store timestamps for video frames,
+ * but only store a framerate, like Ogg and AVI.
+ * </para>
+ * <para>
+ * A conversion to a specific framerate can be forced by using filtered caps on
+ * the source pad.
+ * </para>
+ * <para>
+ * The properties "in", "out", "duplicate" and "drop" can be read to obtain
+ * information about number of input frames, output frames, dropped frames
+ * (i.e. the number of unused input frames) and duplicated frames (i.e. the
+ *  number of times an input frame was duplicated, beside being used normally).
+ *
+ * An input stream that needs no adjustments will thus never have dropped or
+ * duplicated frames.
+ *
+ * When the "silent" property is set to FALSE, a GObject property notification
+ * will be emitted whenever one of the "duplicate" or "drop" values changes.
+ * This can potentially cause performance degradation.
+ * Note that property notification will happen from the streaming thread, so
+ * applications should be prepared for this.
+ * </para>
+ * <title>Example pipelines</title>
+ * <para>
+ * <programlisting>
+ * gst-launch -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! videorate ! video/x-raw-yuv,framerate=15/1 ! xvimagesink
+ * </programlisting>
+ * Decode an Ogg/Theora file and adjust the framerate to 15 fps before playing.
+ * To create the test Ogg/Theora file refer to the documentation of theoraenc.
+ * </para>
+ * <para>
+ * <programlisting>
+ * gst-launch -v v4lsrc ! videorate ! video/x-raw-yuv,framerate=25/2 ! theoraenc ! oggmux ! filesink location=v4l.ogg
+ * </programlisting>
+ * Capture video from a V4L device, and adjust the stream to 12.5 fps before
+ * encoding to Ogg/Theora.
+ * </para>
+  * </refsect2>
+ *
+ * Last reviewed on 2006-09-02 (0.10.11)
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <gst/gst.h>
+#include "gstvideorate.h"
 
-GST_DEBUG_CATEGORY (video_rate_debug);
+GST_DEBUG_CATEGORY_STATIC (video_rate_debug);
 #define GST_CAT_DEFAULT video_rate_debug
 
-#define GST_TYPE_VIDEO_RATE \
-  (gst_video_rate_get_type())
-#define GST_VIDEO_RATE(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VIDEO_RATE,GstVideoRate))
-#define GST_VIDEO_RATE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_RATE,GstVideoRate))
-#define GST_IS_VIDEO_RATE(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_RATE))
-#define GST_IS_VIDEO_RATE_CLASS(obj) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_RATE))
-
-typedef struct _GstVideoRate GstVideoRate;
-typedef struct _GstVideoRateClass GstVideoRateClass;
-
-struct _GstVideoRate
-{
-  GstElement element;
-
-  GstPad *sinkpad, *srcpad;
-
-  /* video state */
-  gint from_rate_numerator, from_rate_denominator;
-  gint to_rate_numerator, to_rate_denominator;
-  guint64 next_ts;              /* Timestamp of next buffer to output */
-  GstBuffer *prevbuf;
-  guint64 prev_ts;              /* Previous buffer timestamp */
-  guint64 in, out, dup, drop;
-
-  /* segment handling */
-  gint64 segment_start;
-  gint64 segment_stop;
-  gint64 segment_accum;
-
-  gboolean silent;
-  gdouble new_pref;
-};
-
-struct _GstVideoRateClass
-{
-  GstElementClass parent_class;
-};
-
 /* elementfactory information */
-static GstElementDetails video_rate_details =
+static const GstElementDetails video_rate_details =
 GST_ELEMENT_DETAILS ("Video rate adjuster",
     "Filter/Effect/Video",
     "Drops/duplicates/adjusts timestamps on video frames to make a perfect stream",
@@ -114,6 +133,9 @@ static GstStaticPadTemplate gst_video_rate_sink_template =
 static void gst_video_rate_base_init (gpointer g_class);
 static void gst_video_rate_class_init (GstVideoRateClass * klass);
 static void gst_video_rate_init (GstVideoRate * videorate);
+
+static void gst_video_rate_swap_prev (GstVideoRate * videorate,
+    GstBuffer * buffer, gint64 time);
 static gboolean gst_video_rate_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_video_rate_chain (GstPad * pad, GstBuffer * buffer);
 
@@ -195,7 +217,7 @@ gst_video_rate_class_init (GstVideoRateClass * klass)
           DEFAULT_SILENT, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, ARG_NEW_PREF,
       g_param_spec_double ("new_pref", "New Pref",
-          "Value indicating how much to prefer new frames",
+          "Value indicating how much to prefer new frames (unused)",
           0.0, 1.0, DEFAULT_NEW_PREF, G_PARAM_READWRITE));
 
   element_class->change_state = gst_video_rate_change_state;
@@ -264,12 +286,14 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
   GstPad *otherpad, *opeer;
   gint rate_numerator, rate_denominator;
 
-  videorate = GST_VIDEO_RATE (GST_PAD_PARENT (pad));
+  videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
+
+  GST_DEBUG ("setcaps called %" GST_PTR_FORMAT, caps);
 
   structure = gst_caps_get_structure (caps, 0);
   if (!gst_structure_get_fraction (structure, "framerate",
           &rate_numerator, &rate_denominator))
-    goto done;
+    goto no_framerate;
 
   if (pad == videorate->srcpad) {
     videorate->to_rate_numerator = rate_numerator;
@@ -297,7 +321,7 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
 
       /* see how we can transform the input caps */
       if (!gst_video_rate_transformcaps (pad, caps, otherpad, &transform))
-        goto done;
+        goto no_transform;
 
       /* see what the peer can do */
       peercaps = gst_pad_get_caps (opeer);
@@ -332,36 +356,46 @@ gst_video_rate_setcaps (GstPad * pad, GstCaps * caps)
         videorate->from_rate_denominator = rate_denominator;
       }
       gst_pad_set_caps (otherpad, caps);
+      gst_caps_unref (caps);
       ret = TRUE;
     }
     gst_object_unref (opeer);
   }
 done:
+  /* After a setcaps, our caps may have changed. In that case, we can't use
+   * the old buffer, if there was one (it might have different dimensions) */
+  GST_DEBUG ("swapping old buffers");
+  gst_video_rate_swap_prev (videorate, NULL, GST_CLOCK_TIME_NONE);
+
+  gst_object_unref (videorate);
   return ret;
+
+no_framerate:
+  {
+    GST_DEBUG_OBJECT (videorate, "no framerate specified");
+    goto done;
+  }
+no_transform:
+  {
+    GST_DEBUG_OBJECT (videorate, "no framerate transform possible");
+    ret = FALSE;
+    goto done;
+  }
 }
 
 static void
-gst_video_rate_blank_data (GstVideoRate * videorate)
+gst_video_rate_reset (GstVideoRate * videorate)
 {
-  GST_DEBUG ("resetting data");
-  if (videorate->prevbuf)
-    gst_buffer_unref (videorate->prevbuf);
-  videorate->prevbuf = NULL;
+  GST_DEBUG ("resetting internal variables");
 
-  videorate->from_rate_numerator = 0;
-  videorate->from_rate_denominator = 0;
-  videorate->to_rate_numerator = 0;
-  videorate->to_rate_denominator = 0;
   videorate->in = 0;
   videorate->out = 0;
   videorate->drop = 0;
   videorate->dup = 0;
-  videorate->next_ts = G_GINT64_CONSTANT (0);
-  videorate->prev_ts = G_GINT64_CONSTANT (0);
+  videorate->next_ts = GST_CLOCK_TIME_NONE;
+  gst_video_rate_swap_prev (videorate, NULL, 0);
 
-  videorate->segment_start = 0;
-  videorate->segment_stop = 0;
-  videorate->segment_accum = 0;
+  gst_segment_init (&videorate->segment, GST_FORMAT_TIME);
 }
 
 static void
@@ -382,57 +416,141 @@ gst_video_rate_init (GstVideoRate * videorate)
   gst_pad_set_getcaps_function (videorate->srcpad, gst_video_rate_getcaps);
   gst_pad_set_setcaps_function (videorate->srcpad, gst_video_rate_setcaps);
 
-  gst_video_rate_blank_data (videorate);
+  gst_video_rate_reset (videorate);
   videorate->silent = DEFAULT_SILENT;
   videorate->new_pref = DEFAULT_NEW_PREF;
+
+  videorate->from_rate_numerator = 0;
+  videorate->from_rate_denominator = 0;
+  videorate->to_rate_numerator = 0;
+  videorate->to_rate_denominator = 0;
 }
 
+/* flush the oldest buffer */
 static GstFlowReturn
+gst_video_rate_flush_prev (GstVideoRate * videorate)
+{
+  GstFlowReturn res;
+  GstBuffer *outbuf;
+  GstClockTime push_ts;
+
+  if (!videorate->prevbuf)
+    goto eos_before_buffers;
+
+  /* make sure we can write to the metadata */
+  outbuf = gst_buffer_make_metadata_writable
+      (gst_buffer_ref (videorate->prevbuf));
+
+  GST_BUFFER_OFFSET (outbuf) = videorate->out;
+  GST_BUFFER_OFFSET_END (outbuf) = videorate->out + 1;
+
+  /* this is the timestamp we put on the buffer */
+  push_ts = videorate->next_ts;
+
+  videorate->out++;
+  if (videorate->to_rate_numerator) {
+    /* interpolate next expected timestamp in the segment */
+    videorate->next_ts = videorate->segment.accum + videorate->segment.start +
+        gst_util_uint64_scale (videorate->out,
+        videorate->to_rate_denominator * GST_SECOND,
+        videorate->to_rate_numerator);
+    GST_BUFFER_DURATION (outbuf) = videorate->next_ts - push_ts;
+  }
+
+  /* adapt for looping, bring back to time in current segment. */
+  GST_BUFFER_TIMESTAMP (outbuf) = push_ts - videorate->segment.accum;
+
+  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (videorate->srcpad));
+
+  GST_LOG_OBJECT (videorate,
+      "old is best, dup, pushing buffer outgoing ts %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (push_ts));
+
+  res = gst_pad_push (videorate->srcpad, outbuf);
+
+  return res;
+
+  /* WARNINGS */
+eos_before_buffers:
+  {
+    GST_INFO_OBJECT (videorate, "got EOS before any buffer was received");
+    return GST_FLOW_OK;
+  }
+}
+
+static void
+gst_video_rate_swap_prev (GstVideoRate * videorate, GstBuffer * buffer,
+    gint64 time)
+{
+  GST_LOG_OBJECT (videorate, "swap_prev: storing buffer %p in prev", buffer);
+  if (videorate->prevbuf)
+    gst_buffer_unref (videorate->prevbuf);
+  videorate->prevbuf = buffer;
+  videorate->prev_ts = time;
+}
+
+static gboolean
 gst_video_rate_event (GstPad * pad, GstEvent * event)
 {
   GstVideoRate *videorate;
+  gboolean ret;
 
-  videorate = GST_VIDEO_RATE (GST_PAD_PARENT (pad));
+  videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
     {
-      gint64 start, stop, base;
-      gdouble rate;
+      gint64 start, stop, time;
+      gdouble rate, arate;
       gboolean update;
       GstFormat format;
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &base);
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
+          &start, &stop, &time);
 
-      if (format != GST_FORMAT_TIME) {
-        GST_WARNING ("Got discont but doesn't have GST_FORMAT_TIME value");
-      } else {
-        /* 
-           We just want to update the accumulated stream_time.
-         */
-        videorate->segment_accum +=
-            videorate->segment_stop - videorate->segment_start;
-        videorate->segment_start = start;
-        videorate->segment_stop = stop;
-        GST_DEBUG_OBJECT (videorate, "Updated segment_accum:%" GST_TIME_FORMAT
-            " segment_start:%" GST_TIME_FORMAT " segment_stop:%"
-            GST_TIME_FORMAT, GST_TIME_ARGS (videorate->segment_accum),
-            GST_TIME_ARGS (videorate->segment_start),
-            GST_TIME_ARGS (videorate->segment_stop));
-      }
+      if (format != GST_FORMAT_TIME)
+        goto format_error;
 
+      GST_DEBUG_OBJECT (videorate, "handle NEWSEGMENT");
+
+      /* We just want to update the accumulated stream_time  */
+      gst_segment_set_newsegment_full (&videorate->segment, update, rate, arate,
+          format, start, stop, time);
+
+      GST_DEBUG_OBJECT (videorate, "updated segment: %" GST_SEGMENT_FORMAT,
+          &videorate->segment);
       break;
     }
+    case GST_EVENT_EOS:
+      /* flush last queued frame */
+      GST_DEBUG_OBJECT (videorate, "Got EOS");
+      gst_video_rate_flush_prev (videorate);
+      break;
     case GST_EVENT_FLUSH_STOP:
-    {
-      gst_video_rate_blank_data (videorate);
-    }
+      /* also resets the segment */
+      GST_DEBUG_OBJECT (videorate, "Got FLUSH_STOP");
+      gst_video_rate_reset (videorate);
+      break;
     default:
       break;
   }
 
-  return gst_pad_event_default (pad, event);
+  ret = gst_pad_push_event (videorate->srcpad, event);
+
+done:
+  gst_object_unref (videorate);
+
+  return ret;
+
+  /* ERRORS */
+format_error:
+  {
+    GST_WARNING_OBJECT (videorate,
+        "Got segment but doesn't have GST_FORMAT_TIME value");
+    gst_event_unref (event);
+    ret = FALSE;
+    goto done;
+  }
 }
 
 static GstFlowReturn
@@ -440,31 +558,42 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstVideoRate *videorate;
   GstFlowReturn res = GST_FLOW_OK;
+  GstClockTime intime, in_ts;
 
   videorate = GST_VIDEO_RATE (GST_PAD_PARENT (pad));
 
-  if (videorate->from_rate_numerator == 0 ||
-      videorate->from_rate_denominator == 0 ||
-      videorate->to_rate_denominator == 0 || videorate->to_rate_numerator == 0)
-    return GST_FLOW_NOT_NEGOTIATED;
+  /* make sure the denominators are not 0 */
+  if (videorate->from_rate_denominator == 0 ||
+      videorate->to_rate_denominator == 0)
+    goto not_negotiated;
 
-  /* pull in 2 buffers */
+  in_ts = GST_BUFFER_TIMESTAMP (buffer);
+
+  if (G_UNLIKELY (in_ts == GST_CLOCK_TIME_NONE))
+    goto invalid_buffer;
+
+  GST_DEBUG_OBJECT (videorate, "got buffer with timestamp %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (in_ts));
+
+  /* the input time is the time in the segment + all previously accumulated
+   * segments */
+  intime = in_ts + videorate->segment.accum;
+
+  /* we need to have two buffers to compare */
   if (videorate->prevbuf == NULL) {
-    /* We're sure it's a GstBuffer here */
-    videorate->prevbuf = buffer;
-    videorate->prev_ts =
-        GST_BUFFER_TIMESTAMP (buffer) - videorate->segment_start +
-        videorate->segment_accum;
-    videorate->next_ts = 0;
+    gst_video_rate_swap_prev (videorate, buffer, intime);
+    videorate->in++;
+    if (!GST_CLOCK_TIME_IS_VALID (videorate->next_ts)) {
+      /* new buffer, we expect to output a buffer that matches the first
+       * timestamp in the segment */
+      videorate->next_ts = videorate->segment.start;
+    }
   } else {
-    GstClockTime prevtime, intime;
+    GstClockTime prevtime;
     gint count = 0;
     gint64 diff1, diff2;
 
     prevtime = videorate->prev_ts;
-    intime =
-        GST_BUFFER_TIMESTAMP (buffer) - videorate->segment_start +
-        videorate->segment_accum;
 
     GST_LOG_OBJECT (videorate,
         "BEGINNING prev buf %" GST_TIME_FORMAT " new buf %" GST_TIME_FORMAT
@@ -473,8 +602,23 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 
     videorate->in++;
 
+    /* drop new buffer if it's before previous one */
+    if (intime < prevtime) {
+      GST_DEBUG_OBJECT (videorate,
+          "The new buffer (%" GST_TIME_FORMAT
+          ") is before the previous buffer (%"
+          GST_TIME_FORMAT "). Dropping new buffer.",
+          GST_TIME_ARGS (intime), GST_TIME_ARGS (prevtime));
+      videorate->drop++;
+      if (!videorate->silent)
+        g_object_notify (G_OBJECT (videorate), "drop");
+      gst_buffer_unref (buffer);
+      goto done;
+    }
+
     /* got 2 buffers, see which one is the best */
     do {
+
       diff1 = prevtime - videorate->next_ts;
       diff2 = intime - videorate->next_ts;
 
@@ -492,40 +636,13 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 
       /* output first one when its the best */
       if (diff1 < diff2) {
-        GstBuffer *outbuf;
-        GstClockTime push_ts;
-
         count++;
-        outbuf =
-            gst_buffer_create_sub (videorate->prevbuf, 0,
-            GST_BUFFER_SIZE (videorate->prevbuf));
-        GST_BUFFER_TIMESTAMP (outbuf) = videorate->next_ts;
-        push_ts = GST_BUFFER_TIMESTAMP (outbuf);
-        videorate->out++;
-        if (videorate->to_rate_numerator) {
-          videorate->next_ts =
-              gst_util_uint64_scale_int (videorate->out * GST_SECOND,
-              videorate->to_rate_denominator, videorate->to_rate_numerator);
-          GST_BUFFER_DURATION (outbuf) =
-              videorate->next_ts - GST_BUFFER_TIMESTAMP (outbuf);
-        }
-        /* adapt for looping */
-        GST_BUFFER_TIMESTAMP (outbuf) -= videorate->segment_accum;
-        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (videorate->srcpad));
 
-        GST_LOG_OBJECT (videorate,
-            "old is best, dup, pushing buffer outgoing ts %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (push_ts));
-
-        if ((res = gst_pad_push (videorate->srcpad, outbuf)) != GST_FLOW_OK) {
-          GST_WARNING_OBJECT (videorate, "couldn't push buffer on srcpad:%d",
-              res);
+        /* on error the _flush function posted a warning already */
+        if ((res = gst_video_rate_flush_prev (videorate)) != GST_FLOW_OK) {
+          gst_buffer_unref (buffer);
           goto done;
         }
-
-        GST_LOG_OBJECT (videorate,
-            "old is best, dup, pushed buffer outgoing ts %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (push_ts));
       }
       /* continue while the first one was the best */
     }
@@ -540,8 +657,10 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
     /* if we didn't output the first buffer, we have a drop */
     else if (count == 0) {
       videorate->drop++;
+
       if (!videorate->silent)
         g_object_notify (G_OBJECT (videorate), "drop");
+
       GST_LOG_OBJECT (videorate,
           "new is best, old never used, drop, outgoing ts %"
           GST_TIME_FORMAT, GST_TIME_ARGS (videorate->next_ts));
@@ -554,15 +673,27 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
         videorate->in, videorate->out, videorate->drop, videorate->dup);
 
     /* swap in new one when it's the best */
-    gst_buffer_unref (videorate->prevbuf);
-    videorate->prevbuf = buffer;
-    videorate->prev_ts =
-        GST_BUFFER_TIMESTAMP (buffer) - videorate->segment_start +
-        videorate->segment_accum;
+    gst_video_rate_swap_prev (videorate, buffer, intime);
   }
 done:
-
   return res;
+
+  /* ERRORS */
+not_negotiated:
+  {
+    GST_WARNING_OBJECT (videorate, "no framerate negotiated");
+    gst_buffer_unref (buffer);
+    res = GST_FLOW_NOT_NEGOTIATED;
+    goto done;
+  }
+
+invalid_buffer:
+  {
+    GST_WARNING_OBJECT (videorate,
+        "Got buffer with GST_CLOCK_TIME_NONE timestamp, discarding it");
+    gst_buffer_unref (buffer);
+    goto done;
+  }
 }
 
 static void
@@ -632,7 +763,7 @@ gst_video_rate_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_video_rate_blank_data (videorate);
+      gst_video_rate_reset (videorate);
       break;
     default:
       break;
